# flake8: noqa
"""Contains a mapping of file extension to MIME types.

Multiple extensions can be mapped to the same MIME type Through
http://svn.apache.org/repos/asf/httpd/httpd/trunk/docs/conf/mime.types
"""
EXT_MIME_MAP = {
    "123": "application/vnd.lotus-1-2-3",
    "3dml": "text/vnd.in3d.3dml",
    "3ds": "image/x-3ds",
    "3g2": "video/3gpp2",
    "3gp": "video/3gpp",
    "7z": "application/x-7z-compressed",
    "aab": "application/x-authorware-bin",
    "aac": "audio/x-aac",
    "aam": "application/x-authorware-map",
    "aas": "application/x-authorware-seg",
    "abw": "application/x-abiword",
    "ac": "application/pkix-attr-cert",
    "acc": "application/vnd.americandynamics.acc",
    "ace": "application/x-ace-compressed",
    "acu": "application/vnd.acucobol",
    "acutc": "application/vnd.acucorp",
    "adp": "audio/adpcm",
    "aep": "application/vnd.audiograph",
    "afm": "application/x-font-type1",
    "afp": "application/vnd.ibm.modcap",
    "ahead": "application/vnd.ahead.space",
    "ai": "application/postscript",
    "aif": "audio/x-aiff",
    "aifc": "audio/x-aiff",
    "aiff": "audio/x-aiff",
    "air": "application/vnd.adobe.air-application-installer-package+zip",
    "ait": "application/vnd.dvb.ait",
    "ami": "application/vnd.amiga.ami",
    "apk": "application/vnd.android.package-archive",
    "appcache": "text/cache-manifest",
    "application": "application/x-ms-application",
    "apr": "application/vnd.lotus-approach",
    "arc": "application/x-freearc",
    "asc": "application/pgp-signature",
    "asf": "video/x-ms-asf",
    "asm": "text/x-asm",
    "aso": "application/vnd.accpac.simply.aso",
    "asx": "video/x-ms-asf",
    "atc": "application/vnd.acucorp",
    "atom": "application/atom+xml",
    "atomcat": "application/atomcat+xml",
    "atomsvc": "application/atomsvc+xml",
    "atx": "application/vnd.antix.game-component",
    "au": "audio/basic",
    "avi": "video/x-msvideo",
    "aw": "application/applixware",
    "azf": "application/vnd.airzip.filesecure.azf",
    "azs": "application/vnd.airzip.filesecure.azs",
    "azw": "application/vnd.amazon.ebook",
    "bat": "application/x-msdownload",
    "bcpio": "application/x-bcpio",
    "bdf": "application/x-font-bdf",
    "bdm": "application/vnd.syncml.dm+wbxml",
    "bed": "application/vnd.realvnc.bed",
    "bh2": "application/vnd.fujitsu.oasysprs",
    "bin": "application/octet-stream",
    "blb": "application/x-blorb",
    "blorb": "application/x-blorb",
    "bmi": "application/vnd.bmi",
    "bmp": "image/bmp",
    "book": "application/vnd.framemaker",
    "box": "application/vnd.previewsystems.box",
    "boz": "application/x-bzip2",
    "bpk": "application/octet-stream",
    "btif": "image/prs.btif",
    "bz2": "application/x-bzip2",
    "bz": "application/x-bzip",
    "c11amc": "application/vnd.cluetrust.cartomobile-config",
    "c11amz": "application/vnd.cluetrust.cartomobile-config-pkg",
    "c4d": "application/vnd.clonk.c4group",
    "c4f": "application/vnd.clonk.c4group",
    "c4g": "application/vnd.clonk.c4group",
    "c4p": "application/vnd.clonk.c4group",
    "c4u": "application/vnd.clonk.c4group",
    "cab": "application/vnd.ms-cab-compressed",
    "caf": "audio/x-caf",
    "cap": "application/vnd.tcpdump.pcap",
    "car": "application/vnd.curl.car",
    "cat": "application/vnd.ms-pki.seccat",
    "cb7": "application/x-cbr",
    "cba": "application/x-cbr",
    "cbr": "application/x-cbr",
    "cbt": "application/x-cbr",
    "cbz": "application/x-cbr",
    "cct": "application/x-director",
    "cc": "text/x-c",
    "ccxml": "application/ccxml+xml",
    "cdbcmsg": "application/vnd.contact.cmsg",
    "cdf": "application/x-netcdf",
    "cdkey": "application/vnd.mediastation.cdkey",
    "cdmia": "application/cdmi-capability",
    "cdmic": "application/cdmi-container",
    "cdmid": "application/cdmi-domain",
    "cdmio": "application/cdmi-object",
    "cdmiq": "application/cdmi-queue",
    "cdx": "chemical/x-cdx",
    "cdxml": "application/vnd.chemdraw+xml",
    "cdy": "application/vnd.cinderella",
    "cer": "application/pkix-cert",
    "cfs": "application/x-cfs-compressed",
    "cgm": "image/cgm",
    "chat": "application/x-chat",
    "chm": "application/vnd.ms-htmlhelp",
    "chrt": "application/vnd.kde.kchart",
    "cif": "chemical/x-cif",
    "cii": "application/vnd.anser-web-certificate-issue-initiation",
    "cil": "application/vnd.ms-artgalry",
    "cla": "application/vnd.claymore",
    "class": "application/java-vm",
    "clkk": "application/vnd.crick.clicker.keyboard",
    "clkp": "application/vnd.crick.clicker.palette",
    "clkt": "application/vnd.crick.clicker.template",
    "clkw": "application/vnd.crick.clicker.wordbank",
    "clkx": "application/vnd.crick.clicker",
    "clp": "application/x-msclip",
    "cmc": "application/vnd.cosmocaller",
    "cmdf": "chemical/x-cmdf",
    "cml": "chemical/x-cml",
    "cmp": "application/vnd.yellowriver-custom-menu",
    "cmx": "image/x-cmx",
    "cod": "application/vnd.rim.cod",
    "com": "application/x-msdownload",
    "conf": "text/plain",
    "cpio": "application/x-cpio",
    "cpp": "text/x-c",
    "cpt": "application/mac-compactpro",
    "crd": "application/x-mscardfile",
    "crl": "application/pkix-crl",
    "crt": "application/x-x509-ca-cert",
    "cryptonote": "application/vnd.rig.cryptonote",
    "csh": "application/x-csh",
    "csml": "chemical/x-csml",
    "csp": "application/vnd.commonspace",
    "css": "text/css",
    "cst": "application/x-director",
    "csv": "text/csv",
    "c": "text/x-c",
    "cu": "application/cu-seeme",
    "curl": "text/vnd.curl",
    "cww": "application/prs.cww",
    "cxt": "application/x-director",
    "cxx": "text/x-c",
    "dae": "model/vnd.collada+xml",
    "daf": "application/vnd.mobius.daf",
    "dart": "application/vnd.dart",
    "dataless": "application/vnd.fdsn.seed",
    "davmount": "application/davmount+xml",
    "dbk": "application/docbook+xml",
    "dcr": "application/x-director",
    "dcurl": "text/vnd.curl.dcurl",
    "dd2": "application/vnd.oma.dd2+xml",
    "ddd": "application/vnd.fujixerox.ddd",
    "deb": "application/x-debian-package",
    "def": "text/plain",
    "deploy": "application/octet-stream",
    "der": "application/x-x509-ca-cert",
    "dfac": "application/vnd.dreamfactory",
    "dgc": "application/x-dgc-compressed",
    "dic": "text/x-c",
    "dir": "application/x-director",
    "dis": "application/vnd.mobius.dis",
    "dist": "application/octet-stream",
    "distz": "application/octet-stream",
    "djv": "image/vnd.djvu",
    "djvu": "image/vnd.djvu",
    "dll": "application/x-msdownload",
    "dmg": "application/x-apple-diskimage",
    "dmp": "application/vnd.tcpdump.pcap",
    "dms": "application/octet-stream",
    "dna": "application/vnd.dna",
    "doc": "application/msword",
    "docm": "application/vnd.ms-word.document.macroenabled.12",
    "docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "dot": "application/msword",
    "dotm": "application/vnd.ms-word.template.macroenabled.12",
    "dotx": "application/vnd.openxmlformats-officedocument.wordprocessingml.template",
    "dp": "application/vnd.osgi.dp",
    "dpg": "application/vnd.dpgraph",
    "dra": "audio/vnd.dra",
    "dsc": "text/prs.lines.tag",
    "dssc": "application/dssc+der",
    "dtb": "application/x-dtbook+xml",
    "dtd": "application/xml-dtd",
    "dts": "audio/vnd.dts",
    "dtshd": "audio/vnd.dts.hd",
    "dump": "application/octet-stream",
    "dvb": "video/vnd.dvb.file",
    "dvi": "application/x-dvi",
    "dwf": "model/vnd.dwf",
    "dwg": "image/vnd.dwg",
    "dxf": "image/vnd.dxf",
    "dxp": "application/vnd.spotfire.dxp",
    "dxr": "application/x-director",
    "ecelp4800": "audio/vnd.nuera.ecelp4800",
    "ecelp7470": "audio/vnd.nuera.ecelp7470",
    "ecelp9600": "audio/vnd.nuera.ecelp9600",
    "ecma": "application/ecmascript",
    "edm": "application/vnd.novadigm.edm",
    "edx": "application/vnd.novadigm.edx",
    "efif": "application/vnd.picsel",
    "ei6": "application/vnd.pg.osasli",
    "elc": "application/octet-stream",
    "emf": "application/x-msmetafile",
    "eml": "message/rfc822",
    "emma": "application/emma+xml",
    "emz": "application/x-msmetafile",
    "eol": "audio/vnd.digital-winds",
    "eot": "application/vnd.ms-fontobject",
    "eps": "application/postscript",
    "epub": "application/epub+zip",
    "es3": "application/vnd.eszigno3+xml",
    "esa": "application/vnd.osgi.subsystem",
    "esf": "application/vnd.epson.esf",
    "et3": "application/vnd.eszigno3+xml",
    "etx": "text/x-setext",
    "eva": "application/x-eva",
    "evy": "application/x-envoy",
    "exe": "application/x-msdownload",
    "exi": "application/exi",
    "ext": "application/vnd.novadigm.ext",
    "ez2": "application/vnd.ezpix-album",
    "ez3": "application/vnd.ezpix-package",
    "ez": "application/andrew-inset",
    "f4v": "video/x-f4v",
    "f77": "text/x-fortran",
    "f90": "text/x-fortran",
    "fbs": "image/vnd.fastbidsheet",
    "fcdt": "application/vnd.adobe.formscentral.fcdt",
    "fcs": "application/vnd.isac.fcs",
    "fdf": "application/vnd.fdf",
    "fe_launch": "application/vnd.denovo.fcselayout-link",
    "fg5": "application/vnd.fujitsu.oasysgp",
    "fgd": "application/x-director",
    "fh4": "image/x-freehand",
    "fh5": "image/x-freehand",
    "fh7": "image/x-freehand",
    "fhc": "image/x-freehand",
    "fh": "image/x-freehand",
    "fig": "application/x-xfig",
    "flac": "audio/x-flac",
    "fli": "video/x-fli",
    "flo": "application/vnd.micrografx.flo",
    "flv": "video/x-flv",
    "flw": "application/vnd.kde.kivio",
    "flx": "text/vnd.fmi.flexstor",
    "fly": "text/vnd.fly",
    "fm": "application/vnd.framemaker",
    "fnc": "application/vnd.frogans.fnc",
    "for": "text/x-fortran",
    "fpx": "image/vnd.fpx",
    "frame": "application/vnd.framemaker",
    "fsc": "application/vnd.fsc.weblaunch",
    "fst": "image/vnd.fst",
    "ftc": "application/vnd.fluxtime.clip",
    "f": "text/x-fortran",
    "fti": "application/vnd.anser-web-funds-transfer-initiation",
    "fvt": "video/vnd.fvt",
    "fxp": "application/vnd.adobe.fxp",
    "fxpl": "application/vnd.adobe.fxp",
    "fzs": "application/vnd.fuzzysheet",
    "g2w": "application/vnd.geoplan",
    "g3": "image/g3fax",
    "g3w": "application/vnd.geospace",
    "gac": "application/vnd.groove-account",
    "gam": "application/x-tads",
    "gbr": "application/rpki-ghostbusters",
    "gca": "application/x-gca-compressed",
    "gdl": "model/vnd.gdl",
    "geo": "application/vnd.dynageo",
    "gex": "application/vnd.geometry-explorer",
    "ggb": "application/vnd.geogebra.file",
    "ggs": "application/vnd.geogebra.slides",
    "ggt": "application/vnd.geogebra.tool",
    "ghf": "application/vnd.groove-help",
    "gif": "image/gif",
    "gim": "application/vnd.groove-identity-message",
    "gml": "application/gml+xml",
    "gmx": "application/vnd.gmx",
    "gnumeric": "application/x-gnumeric",
    "gph": "application/vnd.flographit",
    "gpx": "application/gpx+xml",
    "gqf": "application/vnd.grafeq",
    "gqs": "application/vnd.grafeq",
    "gram": "application/srgs",
    "gramps": "application/x-gramps-xml",
    "gre": "application/vnd.geometry-explorer",
    "grv": "application/vnd.groove-injector",
    "grxml": "application/srgs+xml",
    "gsf": "application/x-font-ghostscript",
    "gtar": "application/x-gtar",
    "gtm": "application/vnd.groove-tool-message",
    "gtw": "model/vnd.gtw",
    "gv": "text/vnd.graphviz",
    "gxf": "application/gxf",
    "gxt": "application/vnd.geonext",
    "h261": "video/h261",
    "h263": "video/h263",
    "h264": "video/h264",
    "hal": "application/vnd.hal+xml",
    "hbci": "application/vnd.hbci",
    "hdf": "application/x-hdf",
    "hh": "text/x-c",
    "hlp": "application/winhlp",
    "hpgl": "application/vnd.hp-hpgl",
    "hpid": "application/vnd.hp-hpid",
    "hps": "application/vnd.hp-hps",
    "hqx": "application/mac-binhex40",
    "h": "text/x-c",
    "htke": "application/vnd.kenameaapp",
    "html": "text/html",
    "htm": "text/html",
    "hvd": "application/vnd.yamaha.hv-dic",
    "hvp": "application/vnd.yamaha.hv-voice",
    "hvs": "application/vnd.yamaha.hv-script",
    "i2g": "application/vnd.intergeo",
    "icc": "application/vnd.iccprofile",
    "ice": "x-conference/x-cooltalk",
    "icm": "application/vnd.iccprofile",
    "ico": "image/x-icon",
    "ics": "text/calendar",
    "ief": "image/ief",
    "ifb": "text/calendar",
    "ifm": "application/vnd.shana.informed.formdata",
    "iges": "model/iges",
    "igl": "application/vnd.igloader",
    "igm": "application/vnd.insors.igm",
    "igs": "model/iges",
    "igx": "application/vnd.micrografx.igx",
    "iif": "application/vnd.shana.informed.interchange",
    "imp": "application/vnd.accpac.simply.imp",
    "ims": "application/vnd.ms-ims",
    "ink": "application/inkml+xml",
    "inkml": "application/inkml+xml",
    "install": "application/x-install-instructions",
    "in": "text/plain",
    "iota": "application/vnd.astraea-software.iota",
    "ipfix": "application/ipfix",
    "ipk": "application/vnd.shana.informed.package",
    "irm": "application/vnd.ibm.rights-management",
    "irp": "application/vnd.irepository.package+xml",
    "iso": "application/x-iso9660-image",
    "itp": "application/vnd.shana.informed.formtemplate",
    "ivp": "application/vnd.immervision-ivp",
    "ivu": "application/vnd.immervision-ivu",
    "jad": "text/vnd.sun.j2me.app-descriptor",
    "jam": "application/vnd.jam",
    "jar": "application/java-archive",
    "java": "text/x-java-source",
    "jisp": "application/vnd.jisp",
    "jlt": "application/vnd.hp-jlyt",
    "jnlp": "application/x-java-jnlp-file",
    "joda": "application/vnd.joost.joda-archive",
    "jpeg": "image/jpeg",
    "jpe": "image/jpeg",
    "jpg": "image/jpeg",
    "jpgm": "video/jpm",
    "jpgv": "video/jpeg",
    "jpm": "video/jpm",
    "json": "application/json",
    "jsonml": "application/jsonml+json",
    "js": "text/javascript",
    "kar": "audio/midi",
    "karbon": "application/vnd.kde.karbon",
    "kfo": "application/vnd.kde.kformula",
    "kia": "application/vnd.kidspiration",
    "kml": "application/vnd.google-earth.kml+xml",
    "kmz": "application/vnd.google-earth.kmz",
    "kne": "application/vnd.kinar",
    "knp": "application/vnd.kinar",
    "kon": "application/vnd.kde.kontour",
    "kpr": "application/vnd.kde.kpresenter",
    "kpt": "application/vnd.kde.kpresenter",
    "kpxx": "application/vnd.ds-keypoint",
    "ksp": "application/vnd.kde.kspread",
    "ktr": "application/vnd.kahootz",
    "ktx": "image/ktx",
    "ktz": "application/vnd.kahootz",
    "kwd": "application/vnd.kde.kword",
    "kwt": "application/vnd.kde.kword",
    "lasxml": "application/vnd.las.las+xml",
    "latex": "application/x-latex",
    "lbd": "application/vnd.llamagraphics.life-balance.desktop",
    "lbe": "application/vnd.llamagraphics.life-balance.exchange+xml",
    "les": "application/vnd.hhe.lesson-player",
    "lha": "application/x-lzh-compressed",
    "link66": "application/vnd.route66.link66+xml",
    "list3820": "application/vnd.ibm.modcap",
    "listafp": "application/vnd.ibm.modcap",
    "list": "text/plain",
    "lnk": "application/x-ms-shortcut",
    "log": "text/plain",
    "lostxml": "application/lost+xml",
    "lrf": "application/octet-stream",
    "lrm": "application/vnd.ms-lrm",
    "ltf": "application/vnd.frogans.ltf",
    "lvp": "audio/vnd.lucent.voice",
    "lwp": "application/vnd.lotus-wordpro",
    "lzh": "application/x-lzh-compressed",
    "m13": "application/x-msmediaview",
    "m14": "application/x-msmediaview",
    "m1v": "video/mpeg",
    "m21": "application/mp21",
    "m2a": "audio/mpeg",
    "m2v": "video/mpeg",
    "m3a": "audio/mpeg",
    "m3u8": "application/vnd.apple.mpegurl",
    "m3u": "audio/x-mpegurl",
    "m4a": "audio/mp4",
    "m4u": "video/vnd.mpegurl",
    "m4v": "video/x-m4v",
    "ma": "application/mathematica",
    "mads": "application/mads+xml",
    "mag": "application/vnd.ecowin.chart",
    "maker": "application/vnd.framemaker",
    "man": "text/troff",
    "mar": "application/octet-stream",
    "mathml": "application/mathml+xml",
    "mb": "application/mathematica",
    "mbk": "application/vnd.mobius.mbk",
    "mbox": "application/mbox",
    "mc1": "application/vnd.medcalcdata",
    "mcd": "application/vnd.mcd",
    "mcurl": "text/vnd.curl.mcurl",
    "mdb": "application/x-msaccess",
    "mdi": "image/vnd.ms-modi",
    "mesh": "model/mesh",
    "meta4": "application/metalink4+xml",
    "metalink": "application/metalink+xml",
    "me": "text/troff",
    "mets": "application/mets+xml",
    "mfm": "application/vnd.mfmp",
    "mft": "application/rpki-manifest",
    "mgp": "application/vnd.osgeo.mapguide.package",
    "mgz": "application/vnd.proteus.magazine",
    "mid": "audio/midi",
    "midi": "audio/midi",
    "mie": "application/x-mie",
    "mif": "application/vnd.mif",
    "mime": "message/rfc822",
    "mj2": "video/mj2",
    "mjp2": "video/mj2",
    "mjs": "text/javascript",
    "mk3d": "video/x-matroska",
    "mka": "audio/x-matroska",
    "mks": "video/x-matroska",
    "mkv": "video/x-matroska",
    "mlp": "application/vnd.dolby.mlp",
    "mmd": "application/vnd.chipnuts.karaoke-mmd",
    "mmf": "application/vnd.smaf",
    "mmr": "image/vnd.fujixerox.edmics-mmr",
    "mng": "video/x-mng",
    "mny": "application/x-msmoney",
    "mobi": "application/x-mobipocket-ebook",
    "mods": "application/mods+xml",
    "movie": "video/x-sgi-movie",
    "mov": "video/quicktime",
    "mp21": "application/mp21",
    "mp2a": "audio/mpeg",
    "mp2": "audio/mpeg",
    "mp3": "audio/mpeg",
    "mp4a": "audio/mp4",
    "mp4s": "application/mp4",
    "mp4": "video/mp4",
    "mp4v": "video/mp4",
    "mpc": "application/vnd.mophun.certificate",
    "mpeg": "video/mpeg",
    "mpe": "video/mpeg",
    "mpg4": "video/mp4",
    "mpga": "audio/mpeg",
    "mpg": "video/mpeg",
    "mpkg": "application/vnd.apple.installer+xml",
    "mpm": "application/vnd.blueice.multipass",
    "mpn": "application/vnd.mophun.application",
    "mpp": "application/vnd.ms-project",
    "mpt": "application/vnd.ms-project",
    "mpy": "application/vnd.ibm.minipay",
    "mqy": "application/vnd.mobius.mqy",
    "mrc": "application/marc",
    "mrcx": "application/marcxml+xml",
    "mscml": "application/mediaservercontrol+xml",
    "mseed": "application/vnd.fdsn.mseed",
    "mseq": "application/vnd.mseq",
    "msf": "application/vnd.epson.msf",
    "msh": "model/mesh",
    "msi": "application/x-msdownload",
    "msl": "application/vnd.mobius.msl",
    "ms": "text/troff",
    "msty": "application/vnd.muvee.style",
    "mts": "model/vnd.mts",
    "mus": "application/vnd.musician",
    "musicxml": "application/vnd.recordare.musicxml+xml",
    "mvb": "application/x-msmediaview",
    "mwf": "application/vnd.mfer",
    "mxf": "application/mxf",
    "mxl": "application/vnd.recordare.musicxml",
    "mxml": "application/xv+xml",
    "mxs": "application/vnd.triscape.mxs",
    "mxu": "video/vnd.mpegurl",
    "n3": "text/n3",
    "nb": "application/mathematica",
    "nbp": "application/vnd.wolfram.player",
    "nc": "application/x-netcdf",
    "ncx": "application/x-dtbncx+xml",
    "nfo": "text/x-nfo",
    "n-gage": "application/vnd.nokia.n-gage.symbian.install",
    "ngdat": "application/vnd.nokia.n-gage.data",
    "nitf": "application/vnd.nitf",
    "nlu": "application/vnd.neurolanguage.nlu",
    "nml": "application/vnd.enliven",
    "nnd": "application/vnd.noblenet-directory",
    "nns": "application/vnd.noblenet-sealer",
    "nnw": "application/vnd.noblenet-web",
    "npx": "image/vnd.net-fpx",
    "nsc": "application/x-conference",
    "nsf": "application/vnd.lotus-notes",
    "ntf": "application/vnd.nitf",
    "nzb": "application/x-nzb",
    "oa2": "application/vnd.fujitsu.oasys2",
    "oa3": "application/vnd.fujitsu.oasys3",
    "oas": "application/vnd.fujitsu.oasys",
    "obd": "application/x-msbinder",
    "obj": "application/x-tgif",
    "oda": "application/oda",
    "odb": "application/vnd.oasis.opendocument.database",
    "odc": "application/vnd.oasis.opendocument.chart",
    "odf": "application/vnd.oasis.opendocument.formula",
    "odft": "application/vnd.oasis.opendocument.formula-template",
    "odg": "application/vnd.oasis.opendocument.graphics",
    "odi": "application/vnd.oasis.opendocument.image",
    "odm": "application/vnd.oasis.opendocument.text-master",
    "odp": "application/vnd.oasis.opendocument.presentation",
    "ods": "application/vnd.oasis.opendocument.spreadsheet",
    "odt": "application/vnd.oasis.opendocument.text",
    "oga": "audio/ogg",
    "ogg": "audio/ogg",
    "ogv": "video/ogg",
    "ogx": "application/ogg",
    "omdoc": "application/omdoc+xml",
    "onepkg": "application/onenote",
    "onetmp": "application/onenote",
    "onetoc2": "application/onenote",
    "onetoc": "application/onenote",
    "opf": "application/oebps-package+xml",
    "opml": "text/x-opml",
    "oprc": "application/vnd.palm",
    "opus": "audio/ogg",
    "org": "application/vnd.lotus-organizer",
    "osf": "application/vnd.yamaha.openscoreformat",
    "osfpvg": "application/vnd.yamaha.openscoreformat.osfpvg+xml",
    "otc": "application/vnd.oasis.opendocument.chart-template",
    "otf": "font/otf",
    "otg": "application/vnd.oasis.opendocument.graphics-template",
    "oth": "application/vnd.oasis.opendocument.text-web",
    "oti": "application/vnd.oasis.opendocument.image-template",
    "otp": "application/vnd.oasis.opendocument.presentation-template",
    "ots": "application/vnd.oasis.opendocument.spreadsheet-template",
    "ott": "application/vnd.oasis.opendocument.text-template",
    "oxps": "application/oxps",
    "oxt": "application/vnd.openofficeorg.extension",
    "p10": "application/pkcs10",
    "p12": "application/x-pkcs12",
    "p7b": "application/x-pkcs7-certificates",
    "p7c": "application/pkcs7-mime",
    "p7m": "application/pkcs7-mime",
    "p7r": "application/x-pkcs7-certreqresp",
    "p7s": "application/pkcs7-signature",
    "p8": "application/pkcs8",
    "pas": "text/x-pascal",
    "paw": "application/vnd.pawaafile",
    "pbd": "application/vnd.powerbuilder6",
    "pbm": "image/x-portable-bitmap",
    "pcap": "application/vnd.tcpdump.pcap",
    "pcf": "application/x-font-pcf",
    "pcl": "application/vnd.hp-pcl",
    "pclxl": "application/vnd.hp-pclxl",
    "pct": "image/x-pict",
    "pcurl": "application/vnd.curl.pcurl",
    "pcx": "image/x-pcx",
    "pdb": "application/vnd.palm",
    "pdf": "application/pdf",
    "pfa": "application/x-font-type1",
    "pfb": "application/x-font-type1",
    "pfm": "application/x-font-type1",
    "pfr": "application/font-tdpfr",
    "pfx": "application/x-pkcs12",
    "pgm": "image/x-portable-graymap",
    "pgn": "application/x-chess-pgn",
    "pgp": "application/pgp-encrypted",
    "pic": "image/x-pict",
    "pkg": "application/octet-stream",
    "pki": "application/pkixcmp",
    "pkipath": "application/pkix-pkipath",
    "plb": "application/vnd.3gpp.pic-bw-large",
    "plc": "application/vnd.mobius.plc",
    "plf": "application/vnd.pocketlearn",
    "pls": "application/pls+xml",
    "pml": "application/vnd.ctc-posml",
    "png": "image/png",
    "pnm": "image/x-portable-anymap",
    "portpkg": "application/vnd.macports.portpkg",
    "pot": "application/vnd.ms-powerpoint",
    "potm": "application/vnd.ms-powerpoint.template.macroenabled.12",
    "potx": "application/vnd.openxmlformats-officedocument.presentationml.template",
    "ppam": "application/vnd.ms-powerpoint.addin.macroenabled.12",
    "ppd": "application/vnd.cups-ppd",
    "ppm": "image/x-portable-pixmap",
    "pps": "application/vnd.ms-powerpoint",
    "ppsm": "application/vnd.ms-powerpoint.slideshow.macroenabled.12",
    "ppsx": "application/vnd.openxmlformats-officedocument.presentationml.slideshow",
    "ppt": "application/vnd.ms-powerpoint",
    "pptm": "application/vnd.ms-powerpoint.presentation.macroenabled.12",
    "pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    "pqa": "application/vnd.palm",
    "prc": "application/x-mobipocket-ebook",
    "pre": "application/vnd.lotus-freelance",
    "prf": "application/pics-rules",
    "ps": "application/postscript",
    "psb": "application/vnd.3gpp.pic-bw-small",
    "psd": "image/vnd.adobe.photoshop",
    "psf": "application/x-font-linux-psf",
    "pskcxml": "application/pskc+xml",
    "p": "text/x-pascal",
    "ptid": "application/vnd.pvi.ptid1",
    "pub": "application/x-mspublisher",
    "pvb": "application/vnd.3gpp.pic-bw-var",
    "pwn": "application/vnd.3m.post-it-notes",
    "pya": "audio/vnd.ms-playready.media.pya",
    "pyv": "video/vnd.ms-playready.media.pyv",
    "qam": "application/vnd.epson.quickanime",
    "qbo": "application/vnd.intu.qbo",
    "qfx": "application/vnd.intu.qfx",
    "qps": "application/vnd.publishare-delta-tree",
    "qt": "video/quicktime",
    "qwd": "application/vnd.quark.quarkxpress",
    "qwt": "application/vnd.quark.quarkxpress",
    "qxb": "application/vnd.quark.quarkxpress",
    "qxd": "application/vnd.quark.quarkxpress",
    "qxl": "application/vnd.quark.quarkxpress",
    "qxt": "application/vnd.quark.quarkxpress",
    "ra": "audio/x-pn-realaudio",
    "ram": "audio/x-pn-realaudio",
    "rar": "application/x-rar-compressed",
    "ras": "image/x-cmu-raster",
    "rcprofile": "application/vnd.ipunplugged.rcprofile",
    "rdf": "application/rdf+xml",
    "rdz": "application/vnd.data-vision.rdz",
    "rep": "application/vnd.businessobjects",
    "res": "application/x-dtbresource+xml",
    "rgb": "image/x-rgb",
    "rif": "application/reginfo+xml",
    "rip": "audio/vnd.rip",
    "ris": "application/x-research-info-systems",
    "rl": "application/resource-lists+xml",
    "rlc": "image/vnd.fujixerox.edmics-rlc",
    "rld": "application/resource-lists-diff+xml",
    "rm": "application/vnd.rn-realmedia",
    "rmi": "audio/midi",
    "rmp": "audio/x-pn-realaudio-plugin",
    "rms": "application/vnd.jcp.javame.midlet-rms",
    "rmvb": "application/vnd.rn-realmedia-vbr",
    "rnc": "application/relax-ng-compact-syntax",
    "roa": "application/rpki-roa",
    "roff": "text/troff",
    "rp9": "application/vnd.cloanto.rp9",
    "rpss": "application/vnd.nokia.radio-presets",
    "rpst": "application/vnd.nokia.radio-preset",
    "rq": "application/sparql-query",
    "rs": "application/rls-services+xml",
    "rsd": "application/rsd+xml",
    "rss": "application/rss+xml",
    "rtf": "application/rtf",
    "rtx": "text/richtext",
    "s3m": "audio/s3m",
    "saf": "application/vnd.yamaha.smaf-audio",
    "sbml": "application/sbml+xml",
    "sc": "application/vnd.ibm.secure-container",
    "scd": "application/x-msschedule",
    "scm": "application/vnd.lotus-screencam",
    "scq": "application/scvp-cv-request",
    "scs": "application/scvp-cv-response",
    "scurl": "text/vnd.curl.scurl",
    "sda": "application/vnd.stardivision.draw",
    "sdc": "application/vnd.stardivision.calc",
    "sdd": "application/vnd.stardivision.impress",
    "sdkd": "application/vnd.solent.sdkm+xml",
    "sdkm": "application/vnd.solent.sdkm+xml",
    "sdp": "application/sdp",
    "sdw": "application/vnd.stardivision.writer",
    "see": "application/vnd.seemail",
    "seed": "application/vnd.fdsn.seed",
    "sema": "application/vnd.sema",
    "semd": "application/vnd.semd",
    "semf": "application/vnd.semf",
    "ser": "application/java-serialized-object",
    "setpay": "application/set-payment-initiation",
    "setreg": "application/set-registration-initiation",
    "sfd-hdstx": "application/vnd.hydrostatix.sof-data",
    "sfs": "application/vnd.spotfire.sfs",
    "sfv": "text/x-sfv",
    "sgi": "image/sgi",
    "sgl": "application/vnd.stardivision.writer-global",
    "sgml": "text/sgml",
    "sgm": "text/sgml",
    "sh": "application/x-sh",
    "shar": "application/x-shar",
    "shf": "application/shf+xml",
    "sid": "image/x-mrsid-image",
    "sig": "application/pgp-signature",
    "sil": "audio/silk",
    "silo": "model/mesh",
    "sis": "application/vnd.symbian.install",
    "sisx": "application/vnd.symbian.install",
    "sit": "application/x-stuffit",
    "sitx": "application/x-stuffitx",
    "skd": "application/vnd.koan",
    "skm": "application/vnd.koan",
    "skp": "application/vnd.koan",
    "skt": "application/vnd.koan",
    "sldm": "application/vnd.ms-powerpoint.slide.macroenabled.12",
    "sldx": "application/vnd.openxmlformats-officedocument.presentationml.slide",
    "slt": "application/vnd.epson.salt",
    "sm": "application/vnd.stepmania.stepchart",
    "smf": "application/vnd.stardivision.math",
    "smi": "application/smil+xml",
    "smil": "application/smil+xml",
    "smv": "video/x-smv",
    "smzip": "application/vnd.stepmania.package",
    "snd": "audio/basic",
    "snf": "application/x-font-snf",
    "so": "application/octet-stream",
    "spc": "application/x-pkcs7-certificates",
    "spf": "application/vnd.yamaha.smaf-phrase",
    "spl": "application/x-futuresplash",
    "spot": "text/vnd.in3d.spot",
    "spp": "application/scvp-vp-response",
    "spq": "application/scvp-vp-request",
    "spx": "audio/ogg",
    "sql": "application/x-sql",
    "src": "application/x-wais-source",
    "srt": "application/x-subrip",
    "sru": "application/sru+xml",
    "srx": "application/sparql-results+xml",
    "ssdl": "application/ssdl+xml",
    "sse": "application/vnd.kodak-descriptor",
    "ssf": "application/vnd.epson.ssf",
    "ssml": "application/ssml+xml",
    "st": "application/vnd.sailingtracker.track",
    "stc": "application/vnd.sun.xml.calc.template",
    "std": "application/vnd.sun.xml.draw.template",
    "s": "text/x-asm",
    "stf": "application/vnd.wt.stf",
    "sti": "application/vnd.sun.xml.impress.template",
    "stk": "application/hyperstudio",
    "stl": "application/vnd.ms-pki.stl",
    "str": "application/vnd.pg.format",
    "stw": "application/vnd.sun.xml.writer.template",
    "sub": "text/vnd.dvb.subtitle",
    "sus": "application/vnd.sus-calendar",
    "susp": "application/vnd.sus-calendar",
    "sv4cpio": "application/x-sv4cpio",
    "sv4crc": "application/x-sv4crc",
    "svc": "application/vnd.dvb.service",
    "svd": "application/vnd.svd",
    "svg": "image/svg+xml",
    "svgz": "image/svg+xml",
    "swa": "application/x-director",
    "swf": "application/x-shockwave-flash",
    "swi": "application/vnd.aristanetworks.swi",
    "sxc": "application/vnd.sun.xml.calc",
    "sxd": "application/vnd.sun.xml.draw",
    "sxg": "application/vnd.sun.xml.writer.global",
    "sxi": "application/vnd.sun.xml.impress",
    "sxm": "application/vnd.sun.xml.math",
    "sxw": "application/vnd.sun.xml.writer",
    "t3": "application/x-t3vm-image",
    "taglet": "application/vnd.mynfc",
    "tao": "application/vnd.tao.intent-module-archive",
    "tar": "application/x-tar",
    "tcap": "application/vnd.3gpp2.tcap",
    "tcl": "application/x-tcl",
    "teacher": "application/vnd.smart.teacher",
    "tei": "application/tei+xml",
    "teicorpus": "application/tei+xml",
    "tex": "application/x-tex",
    "texi": "application/x-texinfo",
    "texinfo": "application/x-texinfo",
    "text": "text/plain",
    "tfi": "application/thraud+xml",
    "tfm": "application/x-tex-tfm",
    "tga": "image/x-tga",
    "thmx": "application/vnd.ms-officetheme",
    "tiff": "image/tiff",
    "tif": "image/tiff",
    "tmo": "application/vnd.tmobile-livetv",
    "torrent": "application/x-bittorrent",
    "tpl": "application/vnd.groove-tool-template",
    "tpt": "application/vnd.trid.tpt",
    "tra": "application/vnd.trueapp",
    "trm": "application/x-msterminal",
    "tr": "text/troff",
    "tsd": "application/timestamped-data",
    "tsv": "text/tab-separated-values",
    "ttc": "font/collection",
    "t": "text/troff",
    "ttf": "font/ttf",
    "ttl": "text/turtle",
    "twd": "application/vnd.simtech-mindmapper",
    "twds": "application/vnd.simtech-mindmapper",
    "txd": "application/vnd.genomatix.tuxedo",
    "txf": "application/vnd.mobius.txf",
    "txt": "text/plain",
    "u32": "application/x-authorware-bin",
    "udeb": "application/x-debian-package",
    "ufd": "application/vnd.ufdl",
    "ufdl": "application/vnd.ufdl",
    "ulx": "application/x-glulx",
    "umj": "application/vnd.umajin",
    "unityweb": "application/vnd.unity",
    "uoml": "application/vnd.uoml+xml",
    "uris": "text/uri-list",
    "uri": "text/uri-list",
    "urls": "text/uri-list",
    "ustar": "application/x-ustar",
    "utz": "application/vnd.uiq.theme",
    "uu": "text/x-uuencode",
    "uva": "audio/vnd.dece.audio",
    "uvd": "application/vnd.dece.data",
    "uvf": "application/vnd.dece.data",
    "uvg": "image/vnd.dece.graphic",
    "uvh": "video/vnd.dece.hd",
    "uvi": "image/vnd.dece.graphic",
    "uvm": "video/vnd.dece.mobile",
    "uvp": "video/vnd.dece.pd",
    "uvs": "video/vnd.dece.sd",
    "uvt": "application/vnd.dece.ttml+xml",
    "uvu": "video/vnd.uvvu.mp4",
    "uvva": "audio/vnd.dece.audio",
    "uvvd": "application/vnd.dece.data",
    "uvvf": "application/vnd.dece.data",
    "uvvg": "image/vnd.dece.graphic",
    "uvvh": "video/vnd.dece.hd",
    "uvvi": "image/vnd.dece.graphic",
    "uvvm": "video/vnd.dece.mobile",
    "uvvp": "video/vnd.dece.pd",
    "uvvs": "video/vnd.dece.sd",
    "uvvt": "application/vnd.dece.ttml+xml",
    "uvvu": "video/vnd.uvvu.mp4",
    "uvv": "video/vnd.dece.video",
    "uvvv": "video/vnd.dece.video",
    "uvvx": "application/vnd.dece.unspecified",
    "uvvz": "application/vnd.dece.zip",
    "uvx": "application/vnd.dece.unspecified",
    "uvz": "application/vnd.dece.zip",
    "vcard": "text/vcard",
    "vcd": "application/x-cdlink",
    "vcf": "text/x-vcard",
    "vcg": "application/vnd.groove-vcard",
    "vcs": "text/x-vcalendar",
    "vcx": "application/vnd.vcx",
    "vis": "application/vnd.visionary",
    "viv": "video/vnd.vivo",
    "vob": "video/x-ms-vob",
    "vor": "application/vnd.stardivision.writer",
    "vox": "application/x-authorware-bin",
    "vrml": "model/vrml",
    "vsd": "application/vnd.visio",
    "vsf": "application/vnd.vsf",
    "vss": "application/vnd.visio",
    "vst": "application/vnd.visio",
    "vsw": "application/vnd.visio",
    "vtu": "model/vnd.vtu",
    "vxml": "application/voicexml+xml",
    "w3d": "application/x-director",
    "wad": "application/x-doom",
    "wasm": "application/wasm",
    "wav": "audio/x-wav",
    "wax": "audio/x-ms-wax",
    "wbmp": "image/vnd.wap.wbmp",
    "wbs": "application/vnd.criticaltools.wbs+xml",
    "wbxml": "application/vnd.wap.wbxml",
    "wcm": "application/vnd.ms-works",
    "wdb": "application/vnd.ms-works",
    "wdp": "image/vnd.ms-photo",
    "weba": "audio/webm",
    "webm": "video/webm",
    "webp": "image/webp",
    "wg": "application/vnd.pmi.widget",
    "wgt": "application/widget",
    "wks": "application/vnd.ms-works",
    "wma": "audio/x-ms-wma",
    "wmd": "application/x-ms-wmd",
    "wmf": "application/x-msmetafile",
    "wmlc": "application/vnd.wap.wmlc",
    "wmlsc": "application/vnd.wap.wmlscriptc",
    "wmls": "text/vnd.wap.wmlscript",
    "wml": "text/vnd.wap.wml",
    "wm": "video/x-ms-wm",
    "wmv": "video/x-ms-wmv",
    "wmx": "video/x-ms-wmx",
    "wmz": "application/x-ms-wmz",
    "woff2": "font/woff2",
    "woff": "font/woff",
    "wpd": "application/vnd.wordperfect",
    "wpl": "application/vnd.ms-wpl",
    "wps": "application/vnd.ms-works",
    "wqd": "application/vnd.wqd",
    "wri": "application/x-mswrite",
    "wrl": "model/vrml",
    "wsdl": "application/wsdl+xml",
    "wspolicy": "application/wspolicy+xml",
    "wtb": "application/vnd.webturbo",
    "wvx": "video/x-ms-wvx",
    "x32": "application/x-authorware-bin",
    "x3db": "model/x3d+binary",
    "x3dbz": "model/x3d+binary",
    "x3d": "model/x3d+xml",
    "x3dv": "model/x3d+vrml",
    "x3dvz": "model/x3d+vrml",
    "x3dz": "model/x3d+xml",
    "xaml": "application/xaml+xml",
    "xap": "application/x-silverlight-app",
    "xar": "application/vnd.xara",
    "xbap": "application/x-ms-xbap",
    "xbd": "application/vnd.fujixerox.docuworks.binder",
    "xbm": "image/x-xbitmap",
    "xdf": "application/xcap-diff+xml",
    "xdm": "application/vnd.syncml.dm+xml",
    "xdp": "application/vnd.adobe.xdp+xml",
    "xdssc": "application/dssc+xml",
    "xdw": "application/vnd.fujixerox.docuworks",
    "xenc": "application/xenc+xml",
    "xer": "application/patch-ops-error+xml",
    "xfdf": "application/vnd.adobe.xfdf",
    "xfdl": "application/vnd.xfdl",
    "xht": "application/xhtml+xml",
    "xhtml": "application/xhtml+xml",
    "xhvml": "application/xv+xml",
    "xif": "image/vnd.xiff",
    "xla": "application/vnd.ms-excel",
    "xlam": "application/vnd.ms-excel.addin.macroenabled.12",
    "xlc": "application/vnd.ms-excel",
    "xlf": "application/x-xliff+xml",
    "xlm": "application/vnd.ms-excel",
    "xls": "application/vnd.ms-excel",
    "xlsb": "application/vnd.ms-excel.sheet.binary.macroenabled.12",
    "xlsm": "application/vnd.ms-excel.sheet.macroenabled.12",
    "xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    "xlt": "application/vnd.ms-excel",
    "xltm": "application/vnd.ms-excel.template.macroenabled.12",
    "xltx": "application/vnd.openxmlformats-officedocument.spreadsheetml.template",
    "xlw": "application/vnd.ms-excel",
    "xm": "audio/xm",
    "xml": "application/xml",
    "xo": "application/vnd.olpc-sugar",
    "xop": "application/xop+xml",
    "xpi": "application/x-xpinstall",
    "xpl": "application/xproc+xml",
    "xpm": "image/x-xpixmap",
    "xpr": "application/vnd.is-xpr",
    "xps": "application/vnd.ms-xpsdocument",
    "xpw": "application/vnd.intercon.formnet",
    "xpx": "application/vnd.intercon.formnet",
    "xsl": "application/xml",
    "xslt": "application/xslt+xml",
    "xsm": "application/vnd.syncml+xml",
    "xspf": "application/xspf+xml",
    "xul": "application/vnd.mozilla.xul+xml",
    "xvm": "application/xv+xml",
    "xvml": "application/xv+xml",
    "xwd": "image/x-xwindowdump",
    "xyz": "chemical/x-xyz",
    "xz": "application/x-xz",
    "yang": "application/yang",
    "yin": "application/yin+xml",
    "z1": "application/x-zmachine",
    "z2": "application/x-zmachine",
    "z3": "application/x-zmachine",
    "z4": "application/x-zmachine",
    "z5": "application/x-zmachine",
    "z6": "application/x-zmachine",
    "z7": "application/x-zmachine",
    "z8": "application/x-zmachine",
    "zaz": "application/vnd.zzazz.deck+xml",
    "zip": "application/zip",
    "zir": "application/vnd.zul",
    "zirz": "application/vnd.zul",
    "zmm": "application/vnd.handheld-entertainment+xml",
}



from typing import Any, Optional

import requests
from unstract.connectors.constants import Common
from unstract.connectors.filesystems import connectors
from unstract.connectors.filesystems.unstract_file_system import (
    AbstractFileSystem,
    UnstractFileSystem,
)
from unstract.sdk.constants import (
    Connector,
    ConnectorKeys,
    ConnectorType,
    LogLevel,
    ToolEnv,
)
from unstract.sdk.platform import PlatformBase
from unstract.sdk.tool.base import BaseTool


class ToolFileSystem(PlatformBase):
    """Class to handle File connectors for Unstract Tools.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(
        self,
        tool: BaseTool,
        platform_host: str,
        platform_port: str,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
            - The platform_host and platform_port are the
                host and port of the platform service.
        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )
        self.fs_connectors = connectors

    def get_fsspec(
        self, tool_instance_id: str, connector_type: str = ConnectorType.OUTPUT
    ) -> Optional[AbstractFileSystem]:
        """Get FsSpec for fileSystem
            1. Get the connection settings from platform service
            using the tool_instance_id
            2. Create UnstractFileSystem based object using the settings
                2.1 Find the type of the database (From Connector Registry)
                2.2 Create the object using the type
                (derived class of UnstractFileSystem)
            3. Send Object.get_fsspec_fs() to the caller

        Args:
            tool_instance_id (str): tool Instance Id
            connector_type (str, optional): _description_.
                Defaults to ConnectorType.OUTPUT.

        Returns:
            Any: _description_
        """
        url = f"{self.base_url}/connector_instance/{Connector.FILE_SYSTEM}"
        query_params = {
            ConnectorKeys.TOOL_INSTANCE_ID: tool_instance_id,
            ConnectorKeys.CONNECTOR_TYPE: connector_type,
        }
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers, params=query_params)
        if response.status_code == 200:
            connector_data: dict[str, Any] = response.json()
            connector_id = connector_data.get(ConnectorKeys.CONNECTOR_ID)
            connector_instance_id = connector_data.get(ConnectorKeys.ID)
            settings = connector_data.get(ConnectorKeys.CONNECTOR_METADATA)
            self.tool.stream_log(
                "Successfully retrieved connector settings "
                f"for connector: {connector_instance_id}"
            )
            if connector_id in self.fs_connectors:
                connector = self.fs_connectors[connector_id][Common.METADATA][
                    Common.CONNECTOR
                ]
                connector_class: UnstractFileSystem = connector(settings)
                return connector_class.get_fsspec_fs()
            else:
                self.tool.stream_log(
                    f"FileSystem not found for connector: {connector_id}",
                    level=LogLevel.ERROR,
                )
                return None

        elif response.status_code == 404:
            self.tool.stream_log(
                f"Connector not found for tool instance {tool_instance_id}",
                level=LogLevel.ERROR,
            )
            return None

        else:
            self.tool.stream_log(
                (
                    f"Error while retrieving connector "
                    "for tool instance: "
                    f"{tool_instance_id} / {response.reason}"
                ),
                level=LogLevel.ERROR,
            )
            return None

    # TODO UN-237: Obtain fsspec object with connector instance ID
    @staticmethod
    def get_fsspec_fs(
        tool: BaseTool,
        tool_instance_id: str,
        fileType: str,
    ) -> AbstractFileSystem:
        """Get fileSystem spec by the help of unstract DB tool.

        Args:
            tool_instance_id (str): ID of the tool instance
            fileType (str): INPUT/OUTPUT
            tool (AbstractTool): Instance of AbstractTool
        Required env variables:
            PLATFORM_HOST: Host of platform service
            PLATFORM_PORT: Port of platform service
        Returns:
            Any: engine
        """
        platform_host = tool.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = tool.get_env_or_die(ToolEnv.PLATFORM_PORT)

        tool.stream_log("Connecting to DB and getting table metadata")
        tool_file_system = ToolFileSystem(
            tool=tool,
            platform_host=platform_host,
            platform_port=platform_port,
        )
        fs: Optional[AbstractFileSystem] = tool_file_system.get_fsspec(
            tool_instance_id, fileType
        )
        if not fs:
            tool.stream_error_and_exit(
                f"FileSystem not found for tool_instance_id {tool_instance_id}"
            )
        return fs



from typing import Any, Optional


class SdkException(Exception):
    def __init__(
        self, *args: Any, user_message: Optional[str] = None, **kwargs: Any
    ) -> None:
        super().__init__(*args, **kwargs)
        self._user_message = user_message

    @property
    def user_message(self) -> Optional[str]:
        return self._user_message

    def __str__(self) -> str:
        return f"{self.message}"



from typing import Any, Optional

import requests
from unstract.sdk.constants import AdapterKeys, LogLevel, ToolEnv
from unstract.sdk.platform import PlatformBase
from unstract.sdk.tool.base import BaseTool


class ToolAdapter(PlatformBase):
    """Class to handle Adapters for Unstract Tools.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(
        self,
        tool: BaseTool,
        platform_host: str,
        platform_port: str,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
            - The platform_host and platform_port are the
                host and port of the platform service.
        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )

    def get_adapter_configuration(
        self,
        adapter_instance_id: str,
    ) -> Optional[dict[str, Any]]:
        """Get Adapter
            1. Get the adapter config from platform service
            using the adapter_instance_id

        Args:
            adapter_instance_id (str): adapter Instance Id

        Returns:
            Any: _description_
        """
        url = f"{self.base_url}/adapter_instance"
        query_params = {AdapterKeys.ADAPTER_INSTANCE_ID: adapter_instance_id}
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers, params=query_params)
        if response.status_code == 200:
            adapter_data: dict[str, Any] = response.json()

            self.tool.stream_log(
                "Successfully retrieved adapter config "
                f"for adapter: {adapter_instance_id}"
            )

            return adapter_data

        elif response.status_code == 404:
            self.tool.stream_log(
                f"adapter not found for: for adapter instance"
                f"{adapter_instance_id}",
                level=LogLevel.ERROR,
            )
            return None

        else:
            self.tool.stream_log(
                (
                    f"Error while retrieving adapter "
                    "for adapter instance: "
                    f"{adapter_instance_id} / {response.reason}"
                ),
                level=LogLevel.ERROR,
            )
            return None

    @staticmethod
    def get_adapter_config(
        tool: BaseTool, adapter_instance_id: str
    ) -> Optional[dict[str, Any]]:
        """Get adapter spec by the help of unstract DB tool.

        Args:
            adapter_instance_id (str): ID of the adapter instance
            tool (AbstractTool): Instance of AbstractTool
        Required env variables:
            PLATFORM_HOST: Host of platform service
            PLATFORM_PORT: Port of platform service
        Returns:
            Any: engine
        """
        platform_host = tool.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = tool.get_env_or_die(ToolEnv.PLATFORM_PORT)

        tool.stream_log("Connecting to DB and getting table metadata")
        tool_adapter = ToolAdapter(
            tool=tool,
            platform_host=platform_host,
            platform_port=platform_port,
        )
        adapter_metadata: Optional[
            dict[str, Any]
        ] = tool_adapter.get_adapter_configuration(adapter_instance_id)
        if not adapter_metadata:
            tool.stream_error_and_exit(
                f"Adapter not found for "
                f"adapter instance: {adapter_instance_id}"
            )
        return adapter_metadata



import os
import shutil
import zipfile
from typing import Optional

import filetype
from llama_index import Document, StorageContext, VectorStoreIndex
from llama_index.node_parser import SimpleNodeParser
from llama_index.vector_stores import VectorStoreQuery, VectorStoreQueryResult

from unstract.sdk.constants import LogLevel, ToolEnv
from unstract.sdk.embedding import ToolEmbedding
from unstract.sdk.exceptions import SdkException
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.utils import ToolUtils
from unstract.sdk.utils.service_context import ServiceContext
from unstract.sdk.vector_db import ToolVectorDB

allowed_pdf_to_text_converters = [
    "default",
    "unstract_llm_whisperer",
    "unstract_camelot",
]


class ToolIndex:
    def __init__(self, tool: BaseTool):
        self.tool = tool

    def get_text_from_index(
        self, embedding_type: str, vector_db: str, doc_id: str
    ):
        embedd_helper = ToolEmbedding(tool=self.tool)
        embedding_li = embedd_helper.get_embedding(
            adapter_instance_id=embedding_type
        )
        if embedding_li is None:
            self.tool.stream_log(
                f"Error loading {embedding_type}", level=LogLevel.ERROR
            )
            raise SdkException(f"Error loading {embedding_type}")
        embedding_dimension = embedd_helper.get_embedding_length(embedding_li)

        vdb_helper = ToolVectorDB(
            tool=self.tool,
        )
        vector_db_li = vdb_helper.get_vector_db(
            adapter_instance_id=vector_db,
            embedding_dimension=embedding_dimension,
        )

        if vector_db_li is None:
            self.tool.stream_log(
                f"Error loading {vector_db}", level=LogLevel.ERROR
            )
            raise SdkException(f"Error loading {vector_db}")

        try:
            self.tool.stream_log(f">>> Querying {vector_db}...")
            self.tool.stream_log(f">>> {doc_id}")
            q = VectorStoreQuery(
                query_embedding=embedding_li.get_query_embedding(" "),
                doc_ids=[doc_id],
                similarity_top_k=10000,
            )
        except Exception as e:
            self.tool.stream_log(
                f"Error querying {vector_db}: {e}", level=LogLevel.ERROR
            )
            raise SdkException(f"Error querying {vector_db}: {e}")

        n: VectorStoreQueryResult = vector_db_li.query(query=q)
        if len(n.nodes) > 0:
            self.tool.stream_log(f"Found {len(n.nodes)} nodes for {doc_id}")
            all_text = ""
            for node in n.nodes:
                all_text += node.get_content()
            return all_text
        else:
            self.tool.stream_log(f"No nodes found for {doc_id}")
            return None

    def _cleanup_text(self, full_text):
        # Remove text which is not requried
        full_text_lines = full_text.split("\n")
        new_context_lines = []
        empty_line_count = 0
        for line in full_text_lines:
            if line.strip() == "":
                empty_line_count += 1
            else:
                if empty_line_count >= 3:
                    empty_line_count = 3
                for i in range(empty_line_count):
                    new_context_lines.append("")
                empty_line_count = 0
                new_context_lines.append(line.rstrip())
        self.tool.stream_log(
            f"Old context length: {len(full_text_lines)}, "
            f"New context length: {len(new_context_lines)}"
        )
        return "\n".join(new_context_lines)

    def index_file(
        self,
        tool_id: str,
        embedding_type: str,
        vector_db: str,
        file_path: str,
        chunk_size: int,
        chunk_overlap: int,
        reindex: bool = False,
        converter: str = "default",
        file_hash: Optional[str] = None,
    ):
        if converter not in allowed_pdf_to_text_converters:
            self.tool.stream_log(
                "pdf-to-text-converters must be one of "
                f"{allowed_pdf_to_text_converters}",
                level=LogLevel.ERROR,
            )
            raise SdkException(
                "pdf-to-text-converters must be one of "
                f"{allowed_pdf_to_text_converters}"
            )

        input_file_type = None
        input_file_type_mime = None

        # Make file content hash if not available
        if not file_hash:
            file_hash = ToolUtils.get_hash_from_file(file_path=file_path)
        with open(file_path, mode="rb") as input_file_obj:
            sample_contents = input_file_obj.read(100)
            input_file_type = filetype.guess(sample_contents)

        if input_file_type is None:
            input_file_type_mime = "text/plain"
        else:
            input_file_type_mime = input_file_type.MIME

        self.tool.stream_log(f"Input file type: {input_file_type_mime}")

        full_text = []

        if input_file_type_mime == "text/plain":
            with open(file_path) as input_file_obj:
                full_text.append(
                    {
                        "section": "full",
                        "text_contents": self._cleanup_text(
                            input_file_obj.read()
                        ),
                    }
                )

        elif input_file_type_mime == "application/pdf":
            raise SdkException(
                "Indexing of PDF files is not supported currently"
            )
            # TODO: Make use of adapters to convert X2Text
            # self.tool.stream_log(f"PDF to text converter: {converter}")
            # if converter == "unstract_llm_whisperer" or converter == "default":  # noqa
            #     full_text.append(
            #         {
            #             "section": "full",
            #             "text_contents": self._cleanup_text(
            #                 x2txt.generate_whisper(
            #                     input_file=file_path,
            #                     mode="text",
            #                     dump_text=True,
            #                 )
            #             ),
            #         }
            #     )
            # else:
            #     # TODO : Support for Camelot
            #     x2txt = X2Text(tool=self.tool)

        elif input_file_type_mime == "application/zip":
            self.tool.stream_log("Zip file extraction required")
            with zipfile.ZipFile(file_path, "r") as zip_ref:
                file_name_from_path = os.path.basename(file_path)
                temp_directory = f"/tmp/unstract_zip/{file_name_from_path}"
                # If temp_directory exists, delete it and create it again
                if os.path.exists(temp_directory):
                    shutil.rmtree(temp_directory)
                os.makedirs(temp_directory)
                zip_ref.extractall(temp_directory)
        else:
            self.tool.stream_log(
                f"Unsupported file type: {input_file_type_mime}",
                level=LogLevel.ERROR,
            )
            raise SdkException(f"Unsupported file type: {input_file_type_mime}")

        doc_id = ToolIndex.generate_file_id(
            tool_id=tool_id,
            file_hash=file_hash,
            vector_db=vector_db,
            embedding=embedding_type,
            chunk_size=chunk_size,
            chunk_overlap=chunk_overlap,
        )

        self.tool.stream_log(f"Checking if doc_id {doc_id} exists")

        vdb_helper = ToolVectorDB(
            tool=self.tool,
        )

        embedd_helper = ToolEmbedding(tool=self.tool)

        embedding_li = embedd_helper.get_embedding(
            adapter_instance_id=embedding_type
        )
        if embedding_li is None:
            self.tool.stream_log(
                f"Error loading {embedding_type}", level=LogLevel.ERROR
            )
            raise SdkException(f"Error loading {embedding_type}")

        embedding_dimension = embedd_helper.get_embedding_length(embedding_li)
        vector_db_li = vdb_helper.get_vector_db(
            adapter_instance_id=vector_db,
            embedding_dimension=embedding_dimension,
        )
        if vector_db_li is None:
            self.tool.stream_log(
                f"Error loading {vector_db}", level=LogLevel.ERROR
            )
            raise SdkException(f"Error loading {vector_db}")

        q = VectorStoreQuery(
            query_embedding=embedding_li.get_query_embedding(" "),
            doc_ids=[doc_id],
        )

        doc_id_not_found = True
        try:
            n: VectorStoreQueryResult = vector_db_li.query(query=q)
            if len(n.nodes) > 0:
                doc_id_not_found = False
                self.tool.stream_log(f"Found {len(n.nodes)} nodes for {doc_id}")
            else:
                self.tool.stream_log(f"No nodes found for {doc_id}")
        except Exception as e:
            self.tool.stream_log(
                f"Error querying {vector_db}: {e}", level=LogLevel.ERROR
            )

        if doc_id_not_found is False and reindex:
            # Delete the nodes for the doc_id
            try:
                vector_db_li.delete(ref_doc_id=doc_id)
                self.tool.stream_log(f"Deleted nodes for {doc_id}")
            except Exception as e:
                self.tool.stream_log(
                    f"Error deleting nodes for {doc_id}: {e}",
                    level=LogLevel.ERROR,
                )
                raise SdkException(f"Error deleting nodes for {doc_id}: {e}")
            doc_id_not_found = True

        if doc_id_not_found:
            # Check if chunking is required
            documents = []
            for item in full_text:
                text = item["text_contents"]
                self.tool.stream_log("Indexing file...")
                document = Document(
                    text=text,
                    doc_id=doc_id,
                    metadata={"section": item["section"]},
                )
                document.id_ = doc_id
                documents.append(document)
            self.tool.stream_log(f"Number of documents: {len(documents)}")
            if chunk_size == 0:
                parser = SimpleNodeParser.from_defaults(
                    chunk_size=len(documents[0].text) + 10, chunk_overlap=0
                )
                nodes = parser.get_nodes_from_documents(
                    documents, show_progress=True
                )
                node = nodes[0]
                node.embedding = embedding_li.get_query_embedding(" ")
                vector_db_li.add(nodes=[node])
                self.tool.stream_log("Added node to vector db")
            else:
                storage_context = StorageContext.from_defaults(
                    vector_store=vector_db_li
                )
                parser = SimpleNodeParser.from_defaults(
                    chunk_size=chunk_size, chunk_overlap=chunk_overlap
                )

                service_context = ServiceContext.get_service_context(
                    platform_api_key=self.tool.get_env_or_die(
                        ToolEnv.PLATFORM_API_KEY
                    ),
                    embed_model=embedding_li,
                    node_parser=parser,
                )

                self.tool.stream_log("Adding nodes to vector db...")
                try:
                    VectorStoreIndex.from_documents(
                        documents,
                        storage_context=storage_context,
                        show_progress=True,
                        service_context=service_context,
                    )
                except Exception as e:
                    self.tool.stream_log(
                        f"Error adding nodes to vector db: {e}",
                        level=LogLevel.ERROR,
                    )
                    raise SdkException(f"Error adding nodes to vector db: {e}")
                self.tool.stream_log("Added nodes to vector db")

        self.tool.stream_log("Done indexing file")
        return doc_id

    @staticmethod
    def generate_file_id(
        tool_id: str,
        file_hash: str,
        vector_db: str,
        embedding: str,
        chunk_size: str,
        chunk_overlap: str,
    ) -> str:
        """Generates a unique ID useful for identifying files during indexing.

        Args:
            tool_id (str): Unique ID of the tool developed / exported
            file_hash (str): Hash of the file contents
            vector_db (str): UUID of the vector DB adapter
            embedding (str): UUID of the embedding adapter
            chunk_size (str): Chunk size for indexing
            chunk_overlap (str): Chunk overlap for indexing

        Returns:
            str: Key representing unique ID for a file
        """
        return (
            f"{tool_id}|{vector_db}|{embedding}|"
            f"{chunk_size}|{chunk_overlap}|{file_hash}"
        )



import requests
from llama_index.callbacks import TokenCountingHandler
from llama_index.callbacks.schema import CBEventType
from unstract.sdk.constants import LogLevel, ToolEnv
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.stream import StreamMixin


class Audit(StreamMixin):
    """The 'Audit' class is responsible for pushing usage data to the platform
    service.

    Methods:
        - push_usage_data: Pushes the usage data to the platform service.

    Attributes:
        None

    Example usage:
        audit = Audit()
        audit.push_usage_data(
            token_counter,
            workflow_id,
            execution_id,
            external_service,
            event_type)
    """

    def __init__(self, log_level: LogLevel = LogLevel.INFO) -> None:
        super().__init__(log_level)

    def push_usage_data(
        self,
        platform_api_key: str,
        token_counter: TokenCountingHandler = None,
        workflow_id: str = "",
        execution_id: str = "",
        external_service: str = "",
        event_type: CBEventType = None,
    ) -> None:
        """Pushes the usage data to the platform service.

        Args:
            token_counter (TokenCountingHandler, optional): The token counter
              object. Defaults to None.
            workflow_id (str, optional): The ID of the workflow. Defaults to "".
            execution_id (str, optional): The ID of the execution. Defaults
              to "".
            external_service (str, optional): The name of the external service.
              Defaults to "".
            event_type (CBEventType, optional): The type of the event. Defaults
              to None.

        Returns:
            None

        Raises:
            requests.RequestException: If there is an error while pushing the
            usage details.
        """
        platform_host = self.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = self.get_env_or_die(ToolEnv.PLATFORM_PORT)

        base_url = SdkHelper.get_platform_base_url(
            platform_host=platform_host, platform_port=platform_port
        )
        bearer_token = platform_api_key

        data = {
            "usage_type": event_type,
            "external_service": external_service,
            "workflow_id": workflow_id,
            "execution_id": execution_id,
            "embedding_tokens": token_counter.total_embedding_token_count,
            "prompt_tokens": token_counter.prompt_llm_token_count,
            "completion_tokens": token_counter.completion_llm_token_count,
            "total_tokens": token_counter.total_llm_token_count,
        }

        url = f"{base_url}/usage"
        headers = {"Authorization": f"Bearer {bearer_token}"}

        try:
            response = requests.post(
                url, headers=headers, json=data, timeout=30
            )
            if response.status_code != 200:
                self.stream_log(
                    log=(
                        "Error while pushing usage details: "
                        f"{response.status_code} {response.reason}",
                    ),
                    level=LogLevel.ERROR,
                )
            else:
                self.stream_log("Successfully pushed usage details")

        except requests.RequestException as e:
            self.stream_log(
                log=f"Error while pushing usage details: {e}",
                level=LogLevel.ERROR,
            )



#!/usr/bin/env python
import argparse
import shutil
from importlib.resources import files
from pathlib import Path


def new_tool(args):
    tool_name = args.tool_name
    if tool_name is None:
        print("Tool name is required")
        exit(1)
    location = args.location
    if location is None:
        print("Location is required")
        exit(1)
    overwrite = args.overwrite
    print(f"Creating new tool {tool_name} at {location}")
    # Check if folder exists
    folder = Path(location).joinpath(tool_name)
    if folder.exists():
        if overwrite:
            print("Folder exists, overwriting")
        else:
            print("Folder exists, exiting")
            exit(1)
    else:
        folder.mkdir(parents=True, exist_ok=True)

    source = Path(files("unstract.sdk").joinpath("static/tool_template/v1/"))
    print(f"Copying files from {source} to {folder}")
    # Copy all files in source to folder, recursively
    shutil.copytree(source, folder, dirs_exist_ok=True)


def main() -> None:
    parser = argparse.ArgumentParser(
        prog="Unstract tool generator",
        description="Script to generate a new Unstract tool",
        epilog="Unstract SDK",
    )
    parser.add_argument(
        "--command", type=str, help="Command to execute", required=True
    )
    parser.add_argument(
        "--tool-name", type=str, help="Tool name", required=False
    )
    parser.add_argument(
        "--location",
        type=str,
        help="Director to create the new tool in",
        required=False,
    )
    parser.add_argument(
        "--overwrite",
        help="Overwrite existing tool",
        required=False,
        default=False,
        action="store_true",
    )
    args = parser.parse_args()
    command = str.upper(args.command)

    if command == "NEW":
        try:
            new_tool(args)
        except Exception as e:
            print(f"Error creating new tool: {e}")
            exit(1)
        print("New tool created successfully")
    else:
        print("Command not supported")
        exit(1)


if __name__ == "__main__":
    main()



from typing import Any, Optional

from llama_index.callbacks import TokenCountingHandler
from llama_index.callbacks.base_handler import BaseCallbackHandler
from llama_index.callbacks.schema import CBEventType
from llama_index.core.embeddings.base import BaseEmbedding
from llama_index.llms.llm import LLM
from unstract.sdk.audit import Audit
from unstract.sdk.constants import LogLevel
from unstract.sdk.tool.stream import StreamMixin


class UsageHandler(StreamMixin, BaseCallbackHandler):
    """UsageHandler class is a subclass of BaseCallbackHandler and is
    responsible for handling usage events in the LLM or Embedding models. It
    provides methods for starting and ending traces, as well as handling event
    starts and ends.

    Attributes:
        - token_counter (TokenCountingHandler): The token counter object used
          to count tokens in the LLM or Embedding models.
        - llm_model (LLM): The LLM model object.
        - embed_model (BaseEmbedding): The embedding model object.
        - workflow_id (str): The ID of the workflow.
        - execution_id (str): The ID of the execution.
        - event_starts_to_ignore (Optional[list[CBEventType]]): A list of event
          types to ignore at the start.
        - event_ends_to_ignore (Optional[list[CBEventType]]): A list of event
          types to ignore at the end.
        - verbose (bool): A flag indicating whether to print verbose output.
    """

    def __init__(
        self,
        token_counter: TokenCountingHandler,
        platform_api_key: str,
        llm_model: LLM = None,
        embed_model: BaseEmbedding = None,
        workflow_id: str = "",
        execution_id: str = "",
        event_starts_to_ignore: Optional[list[CBEventType]] = None,
        event_ends_to_ignore: Optional[list[CBEventType]] = None,
        verbose: bool = False,
        log_level: LogLevel = LogLevel.INFO,
    ) -> None:
        self._verbose = verbose
        self.workflow_id = workflow_id
        self.execution_id = execution_id
        self.token_counter = token_counter
        self.llm_model = llm_model
        self.embed_model = embed_model
        self.platform_api_key = platform_api_key
        super().__init__(
            log_level=log_level,  # StreamMixin's args
            event_starts_to_ignore=event_starts_to_ignore or [],
            event_ends_to_ignore=event_ends_to_ignore or [],
        )

    def start_trace(self, trace_id: Optional[str] = None) -> None:
        return

    def end_trace(
        self,
        trace_id: Optional[str] = None,
        trace_map: Optional[dict[str, list[str]]] = None,
    ) -> None:
        return

    def on_event_start(
        self,
        event_type: CBEventType,
        payload: Optional[dict[str, Any]] = None,
        event_id: str = "",
        parent_id: str = "",
        **kwargs: Any,
    ) -> str:
        return event_id

    def on_event_end(
        self,
        event_type: CBEventType,
        payload: Optional[dict[str, Any]] = None,
        event_id: str = "",
        **kwargs: Any,
    ) -> None:
        """Push the usage of  LLM or Embedding to platform service."""
        if (
            event_type == CBEventType.LLM
            and event_type not in self.event_ends_to_ignore
            and payload is not None
        ):
            model_name = self.llm_model.metadata.model_name
            # Need to push the data to via platform service
            self.stream_log(log=f"Pushing llm usage llm for model {model_name}")
            Audit(log_level=self.log_level).push_usage_data(
                platform_api_key=self.platform_api_key,
                token_counter=self.token_counter,
                event_type=event_type,
                external_service=self.llm_model.metadata.model_name,
                workflow_id=self.workflow_id,
                execution_id=self.execution_id,
            )

        elif (
            event_type == CBEventType.EMBEDDING
            and event_type not in self.event_ends_to_ignore
            and payload is not None
        ):
            model_name = self.embed_model.model_name
            # Need to push the data to via platform service
            self.stream_log(log=f"Pushing llm usage llm for model {model_name}")
            Audit(log_level=self.log_level).push_usage_data(
                platform_api_key=self.platform_api_key,
                token_counter=self.token_counter,
                event_type=event_type,
                external_service=self.embed_model.model_name,
                workflow_id=self.workflow_id,
                execution_id=self.execution_id,
            )



import argparse
import shutil
from json import loads
from pathlib import Path
from typing import Any

from unstract.sdk import get_sdk_version
from unstract.sdk.constants import Command
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.tool.validator import ToolValidator


class ToolExecutor:
    """Takes care of executing a tool's intended command."""

    def __init__(self, tool: BaseTool) -> None:
        self.tool = tool

    def execute(self, args: argparse.Namespace) -> None:
        """Executes the tool with the passed arguments.

        Args:
            args (argparse.Namespace): Parsed arguments to execute with
        """
        command = str.upper(args.command)

        if command in Command.static_commands():
            self.tool.handle_static_command(command)
        elif command == Command.RUN:
            self.execute_run(args=args)
        else:
            self.tool.stream_error_and_exit(f"Command {command} not supported")

    def _setup_for_run(self) -> None:
        """Helps initialize tool execution for RUN command."""
        shutil.rmtree(self.tool.get_output_dir(), ignore_errors=True)
        Path(self.tool.get_output_dir()).mkdir(parents=True, exist_ok=True)

    def execute_run(self, args: argparse.Namespace) -> None:
        """Executes the tool's RUN command.

        Args:
            args (argparse.Namespace): Parsed arguments to execute with
        """
        if args.settings is None:
            self.tool.stream_error_and_exit(
                "--settings are required for RUN command"
            )
        settings: dict[str, Any] = loads(args.settings)

        self._setup_for_run()

        validator = ToolValidator(self.tool)
        settings = validator.validate_pre_execution(settings=settings)

        self.tool.stream_log(
            f"Running tool for "
            f"Workflow ID: {self.tool.workflow_id} "
            f"Execution ID: {self.tool.execution_id} "
            f"SDK Version: {get_sdk_version()}"
        )
        self.tool.run(
            settings=settings,
            input_file=self.tool.get_input_file(),
            output_dir=self.tool.get_output_dir(),
        )
        # TODO: Call tool method to validate if output was written



import unittest
from typing import Any
from unittest.mock import Mock, patch

from unstract.sdk.dbs import ToolDB
from unstract.sdk.tool.base import BaseTool


class UnstractToolDBTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            # self.stream_log("Mock tool running")
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    @patch("requests.get")
    def test_get_engine(self, mock_requests_get: Mock) -> None:
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {
            "connector_id": "postgresql|6db35f45-be11-4fd5-80c5-85c48183afbb",
            "connector_metadata": {
                "connection_url": "",
                "database": "test7",
                "host": "localhost",
                "password": "ascon",
                "port": "5432",
                "user": "test",
            },
            "id": "1d393032-87ff-49a2-8f37-8f8dcee7df62",
            "project_id": "3a723424-0070-44c1-a7ec-2976289542d4",
        }
        mock_requests_get.return_value = mock_response
        db = ToolDB(
            tool=self.tool,
            platform_host="http://localhost",
            platform_port="3001",
        )
        tool_instance_id = "6ad6671b-cc3a-4f71-a43c-c88b5046caae"

        engine = db.get_engine(tool_instance_id=tool_instance_id)
        self.assertIsNotNone(engine)


if __name__ == "__main__":
    unittest.main()



from typing import Optional

from llama_index.embeddings.base import BaseEmbedding
from unstract.adapters.constants import Common
from unstract.adapters.embedding import adapters
from unstract.sdk.adapters import ToolAdapter
from unstract.sdk.constants import LogLevel, ToolSettingsKey
from unstract.sdk.tool.base import BaseTool


class ToolEmbedding:
    __TEST_SNIPPET = "Hello, I am Unstract"

    def __init__(self, tool: BaseTool, tool_settings: dict[str, str] = {}):
        self.tool = tool
        self.max_tokens = 1024 * 16
        self.embedding_adapters = adapters
        self.embedding_adapter_instance_id = tool_settings.get(
            ToolSettingsKey.EMBEDDING_ADAPTER_ID
        )
        self.embedding_adapter_id: Optional[str] = None

    def get_embedding(
        self, adapter_instance_id: Optional[str] = None
    ) -> Optional[BaseEmbedding]:
        adapter_instance_id = (
            adapter_instance_id
            if adapter_instance_id
            else self.embedding_adapter_instance_id
        )
        if adapter_instance_id is not None:
            try:
                embedding_config_data = ToolAdapter.get_adapter_config(
                    self.tool, adapter_instance_id
                )
                embedding_adapter_id = embedding_config_data.get(
                    Common.ADAPTER_ID
                )
                self.embedding_adapter_id = embedding_adapter_id
                if embedding_adapter_id in self.embedding_adapters:
                    embedding_adapter = self.embedding_adapters[
                        embedding_adapter_id
                    ][Common.METADATA][Common.ADAPTER]
                    embedding_metadata = embedding_config_data.get(
                        Common.ADAPTER_METADATA
                    )
                    embedding_adapter_class = embedding_adapter(
                        embedding_metadata
                    )
                    return embedding_adapter_class.get_embedding_instance()
                else:
                    return None
            except Exception as e:
                self.tool.stream_log(
                    log=f"Error getting embedding: {e}", level=LogLevel.ERROR
                )
                return None
        else:
            return None

    def get_embedding_length(self, embedding: BaseEmbedding) -> int:
        embedding_list = embedding._get_text_embedding(self.__TEST_SNIPPET)
        embedding_dimension = len(embedding_list)
        return embedding_dimension



import json
from hashlib import md5, sha256
from typing import Any

from unstract.sdk.constants import FileReaderSettings


class ToolUtils:
    """Class containing utility methods."""

    @staticmethod
    def hash_str(string_to_hash: Any, hash_method: str = "sha256") -> str:
        """Computes the hash for a given input string.

        Useful to hash strings needed for caching and other purposes.
        Hash method defaults to "md5"

        Args:
            string_to_hash (str): String to be hashed
            hash_method (str): Hash hash_method to use, supported ones
                - "md5"

        Returns:
            str: Hashed string
        """
        if hash_method == "md5":
            if isinstance(string_to_hash, bytes):
                return str(md5(string_to_hash).hexdigest())
            return str(md5(string_to_hash.encode()).hexdigest())
        elif hash_method == "sha256":
            if isinstance(string_to_hash, (bytes, bytearray)):
                return str(sha256(string_to_hash).hexdigest())
            return str(sha256(string_to_hash.encode()).hexdigest())
        else:
            raise ValueError(f"Unsupported hash_method: {hash_method}")

    @staticmethod
    def get_hash_from_file(file_path: str):
        hashes = []
        chunk_size = FileReaderSettings.FILE_READER_CHUNK_SIZE

        with open(file_path, "rb") as f:
            while True:
                chunk = f.read(chunk_size)
                if not chunk:
                    break  # End of file
                hashes.append(ToolUtils.hash_str(chunk))
        hash_value = ToolUtils.hash_str("".join(hashes))
        return hash_value

    @staticmethod
    def load_json(file_to_load: str) -> dict[str, Any]:
        """Loads and returns a JSON from a file.

        Args:
            file_to_load (str): Path to the file containing JSON

        Returns:
            dict[str, Any]: The JSON loaded from file
        """
        with open(file_to_load, encoding="utf-8") as f:
            loaded_json: dict[str, Any] = json.load(f)
            return loaded_json

    @staticmethod
    def json_to_str(json_to_dump: dict[str, Any]) -> str:
        """Helps convert the JSON to a string. Useful for dumping the JSON to a
        file.

        Args:
            json_to_dump (dict[str, Any]): Input JSON to dump

        Returns:
            str: String representation of the JSON
        """
        compact_json = json.dumps(json_to_dump, separators=(",", ":"))
        return compact_json



from typing import Any, Optional

import requests
from unstract.sdk.constants import LogLevel
from unstract.sdk.platform import PlatformBase
from unstract.sdk.tool.base import BaseTool


class ToolCache(PlatformBase):
    """Class to handle caching for Unstract Tools.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(
        self, tool: BaseTool, platform_host: str, platform_port: int
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): The host of the platform.
            platform_port (int): The port of the platform.

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
            - The platform_host and platform_port are the host and port of
                the platform service.
        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )

    def set(self, key: str, value: str) -> bool:
        """Sets the value for a key in the cache.

        Args:
            key (str): The key.
            value (str): The value.

        Returns:
            bool: Whether the operation was successful.
        """

        url = f"{self.base_url}/cache"
        json = {"key": key, "value": value}
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.post(url, json=json, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(f"Successfully cached data for key: {key}")
            return True
        else:
            self.tool.stream_log(
                f"Error while caching data for key: {key} / {response.reason}",
                level=LogLevel.ERROR,
            )
            return False

    def get(self, key: str) -> Optional[Any]:
        """Gets the value for a key in the cache.

        Args:
            key (str): The key.

        Returns:
            str: The value.
        """

        url = f"{self.base_url}/cache?key={key}"
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully retrieved cached data for key: {key}"
            )
            return response.text
        elif response.status_code == 404:
            self.tool.stream_log(
                f"Data not found for key: {key}", level=LogLevel.WARN
            )
            return None
        else:
            self.tool.stream_log(
                f"Error while retrieving cached data for key: "
                f"{key} / {response.reason}",
                level=LogLevel.ERROR,
            )
            return None

    def delete(self, key: str) -> bool:
        """Deletes the value for a key in the cache.

        Args:
            key (str): The key.

        Returns:
            bool: Whether the operation was successful.
        """
        url = f"{self.base_url}/cache?key={key}"
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.delete(url, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully deleted cached data for key: {key}"
            )
            return True
        else:
            self.tool.stream_log(
                "Error while deleting cached data "
                f"for key: {key} / {response.reason}",
                level=LogLevel.ERROR,
            )
            return False



class SdkHelper:
    def __init__(self) -> None:
        pass

    @staticmethod
    def get_platform_base_url(platform_host: str, platform_port: str) -> str:
        """Make base url from host and port.

        Args:
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Returns:
            str: URL to the platform service
        """
        if platform_host[-1] == "/":
            return f"{platform_host[:-1]}:{platform_port}"
        return f"{platform_host}:{platform_port}"



from typing import Any, Optional

import requests
from unstract.sdk.constants import LogLevel, PromptStudioKeys, ToolEnv
from unstract.sdk.helper import SdkHelper
from unstract.sdk.tool.base import BaseTool


class PromptTool:
    """Class to handle prompt service methods for Unstract Tools."""

    def __init__(
        self,
        tool: BaseTool,
        prompt_host: str,
        prompt_port: str,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            prompt_host (str): Host of platform service
            prompt_host (str): Port of platform service

        """
        self.tool = tool
        self.base_url = SdkHelper.get_platform_base_url(
            prompt_host, prompt_port
        )
        self.bearer_token = tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)

    def answer_prompt(self, payload: dict) -> dict:
        """Invokes and communicates to prompt service to fetch response for the
        prompt.

        Args:
            file_name (str): File in which the prompt is processed
            outputs (dict): dict of all input data for the tool
            tool_id (str): Unique ID of the tool to be processed

        Returns:
            Sample return dict:
            {
                "status": "OK",
                "error": "",
                "cost": 0,
                structure_output : {}
            }
        """
        result = {
            "status": "ERROR",
            "error": "",
            "cost": 0,
            "structure_output": "",
        }
        # TODO : Implement authorization for prompt service
        # headers = {"Authorization": f"Bearer {self.bearer_token}"}
        # Upload file to platform
        url = f"{self.base_url}/answer-prompt"
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        try:
            response = requests.post(url, json=payload, headers=headers)
            if response.status_code != 200:
                self.tool.stream_log(
                    f"Error while fetching response: {response.text}",
                    level=LogLevel.ERROR,
                )
                result["error"] = response.text
                return result
            else:
                result["status"] = "OK"
                result["structure_output"] = response.text
        except Exception as e:
            self.tool.stream_log(
                f"Error while fetching response for prompt: {e}",
                level=LogLevel.ERROR,
            )
            result["error"] = str(e)
            return result
        return result

    @staticmethod
    def get_exported_tool(
        tool: BaseTool, prompt_registry_id: str
    ) -> Optional[dict[str, Any]]:
        """Get exported custom tool by the help of unstract DB tool.

        Args:
            prompt_registry_id (str): ID of the prompt_registry_id
            tool (AbstractTool): Instance of AbstractTool
        Required env variables:
            PLATFORM_HOST: Host of platform service
            PLATFORM_PORT: Port of platform service
        """
        platform_host = tool.get_env_or_die(ToolEnv.PLATFORM_HOST)
        platform_port = tool.get_env_or_die(ToolEnv.PLATFORM_PORT)

        tool.stream_log("Connecting to DB and getting exported tool metadata")
        base_url = SdkHelper.get_platform_base_url(platform_host, platform_port)
        bearer_token = tool.get_env_or_die(ToolEnv.PLATFORM_API_KEY)

        url = f"{base_url}/custom_tool_instance"
        query_params = {PromptStudioKeys.PROMPT_REGISTRY_ID: prompt_registry_id}
        headers = {"Authorization": f"Bearer {bearer_token}"}
        response = requests.get(url, headers=headers, params=query_params)
        if response.status_code == 200:
            adapter_data: dict[str, Any] = response.json()
            tool.stream_log(
                "Successfully retrieved metadata for the exported "
                f"tool: {prompt_registry_id}"
            )
            return adapter_data

        elif response.status_code == 404:
            tool.stream_error_and_exit(
                f"Exported tool {prompt_registry_id} is not found"
            )
            return None

        else:
            tool.stream_error_and_exit(
                f"Error while retrieving tool metadata "
                "for the exported tool: "
                f"{prompt_registry_id} / {response.reason}"
            )
            return None



import json
import logging
import os
import unittest

from dotenv import load_dotenv
from llama_index import MockEmbedding
from llama_index.vector_stores.types import BasePydanticVectorStore, VectorStore
from parameterized import parameterized
from unstract.adapters.vectordb.helper import VectorDBHelper
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.vector_db import ToolVectorDB

load_dotenv()

logger = logging.getLogger(__name__)


def get_test_values(env_key: str) -> list[str]:
    test_values = json.loads(os.environ.get(env_key))
    return test_values


class ToolVectorDBTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
        ) -> None:
            self.stream_log("Mock tool running")

    def setUp(self) -> None:
        self.tool = self.MockTool()

    @parameterized.expand(
        get_test_values("VECTOR_DB_TEST_VALUES")
        # Works for Qdrant and Postgres
    )
    def test_get_vector_db(self, adapter_instance_id: str) -> None:
        mock_embedding = MockEmbedding(embed_dim=1)
        unstract_tool_vector_db = ToolVectorDB(tool=self.tool)
        vector_store = unstract_tool_vector_db.get_vector_db(
            adapter_instance_id, mock_embedding.embed_dim
        )
        self.assertIsNotNone(vector_store)
        self.assertIsInstance(
            vector_store, (BasePydanticVectorStore, VectorStore)
        )

        result = VectorDBHelper.test_vector_db_instance(vector_store)
        self.assertEqual(result, True)


if __name__ == "__main__":
    unittest.main()



from typing import Any, Optional

import requests
from unstract.connectors.filesystems.unstract_file_system import (
    AbstractFileSystem,
)
from unstract.sdk.constants import LogLevel
from unstract.sdk.platform import PlatformBase
from unstract.sdk.tool.base import BaseTool


class ToolDocs(PlatformBase):
    """Class to handle documents for Unstract Tools.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(self, tool: BaseTool, platform_host: str, platform_port: int):
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): The host of the platform.
            platform_port (int): The port of the platform.

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
            - The platform_host and platform_port are the host and
                port of the platform service.

        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )

    def _upload_file(
        self, file_path: str, fsspec_obj: AbstractFileSystem
    ) -> dict[str, Any]:
        """Uploads a file to the platform service for further processing.

        Args:
            file_path (str): Path to file to upload
            fsspec_obj (AbstractFileSystem): Fsspec object that's configured
                to access the file

        Returns:
            dict: The result of the upload operation.

        Notes:
            Sample return dict:
            {
                "status": "OK",
                "error": "",
                "cost": 0,
                "unique_file_id": "9b44826ff1ed4dfd5dda762776acd4dd"
            }
        """
        result = {
            "status": "ERROR",
            "error": "",
            "cost": 0,
            "unique_file_id": "",
        }
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        # Upload file to platform
        url = f"{self.base_url}/upload"
        try:
            with fsspec_obj.open(file_path, "rb") as file:
                files = {"file": (file_path, file)}
                response = requests.post(url, files=files, headers=headers)
                if response.status_code == 200:
                    self.tool.stream_log(
                        f"Successfully uploaded file: {file_path}"
                    )
                else:
                    self.tool.stream_log(
                        "Error while uploading file: "
                        f"{file_path} / {response.text}",
                        level=LogLevel.ERROR,
                    )
                    result["error"] = response.text
                    return result
                result["status"] = "OK"
                result["unique_file_id"] = response.json()["upload_file_id"]
        except Exception as e:
            self.tool.stream_log(
                f"Error while uploading file: {file_path} / {e}",
                level=LogLevel.ERROR,
            )
            result["error"] = str(e)
            return result
        self.tool.stream_log(f"Uploaded file ID: {result['unique_file_id']}")
        return result

    # def index_file(
    #     self,
    #     workflow_id: str,
    #     embedding_type: str,
    #     vector_db: str,
    #     file_path: str,
    #     fsspec_obj: AbstractFileSystem,
    #     overwrite: bool = False,
    # ) -> dict:
    #     """Indexes a file to the platform.

    #     Args:
    #         workflow_id (str): The workflow id.
    #         embedding_type (str): The embedding type.
    #             Supported values:
    #                 - "Azure OpenAI"
    #         vector_db (str): The vector db.
    #             Supported values:
    #                 - "Postgres pg_vector"
    #         file_path (str): The path to the file to index.
    #         overwrite (bool): Whether to overwrite the file if it already
    #             exists. The default is False.

    #     Returns:
    #         dict: The result of the indexing operation.

    #     Notes:
    #         Sample return dict:
    #         {
    #             "status": "OK",
    #             "error": "",
    #             "cost": 746,
    #             "unique_file_id": "9b44826ff1ed4dfd5dda762776acd4dd"
    #         }
    #     """
    #     result = {"status": "ERROR", "error": "", "cost": 0, "unique_file_id": ""}  # noqa: E501
    #     upload_result = self._upload_file(file_path=file_path, fsspec_obj=fsspec_obj) # noqa: E501
    #     if upload_result["status"] == "ERROR":
    #         return upload_result
    #     upload_file_id = upload_result["unique_file_id"]
    #     self.tool.stream_log(f"Indexing file: {file_path}")

    #     payload = {
    #         "workflow_id": workflow_id,
    #         "upload_file_id": upload_file_id,
    #         "embedding_type": embedding_type,
    #         "vector_db": vector_db,
    #         "file_path": file_path,
    #     }
    #     url = f"{self.base_url}/indexer?overwrite={overwrite}"
    #     headers = {"Authorization": f"Bearer {self.bearer_token}"}
    #     response = requests.post(url, json=payload, headers=headers)
    #     if response.status_code == 200:
    #         self.tool.stream_log(f"Successfully indexed file: {file_path}")
    #         result["status"] = "OK"
    #         result["cost"] = response.json()["cost"]
    #         result["unique_file_id"] = response.json()["unique_file_id"]
    #         return result
    #     else:
    #         self.tool.stream_log(
    #             f"Error while indexing file: {file_path} / {response.text}",
    #             level=LogLevel.ERROR,
    #         )
    #         result["error"] = response.text
    #         return result

    def insert(
        self,
        workflow_id: str,
        unique_file_id: str,
        filename: str,
        filetype: str,
        summary: str,
        embedding_tokens: int,
        llm_tokens: int,
        vector_db: str,
    ):
        """Inserts data for a document into the platform.

        Notes:
            - This method is typically called by the tool's index() method.
                It is not typically called directly.

        Args:
            workflow_id (str): The project id.
            unique_file_id (str): The unique file id.
            filename (str): The filename.
            filetype (str): The filetype. Example: "application/pdf"
            summary (str): The summary. Note: Currently not used.
            embedding_tokens (int): The number of tokens used for the embedding.
            llm_tokens (int): The number of tokens used for the LLM.
            vector_db (str): The vector db.
                Supported values:
                    - "Postgres pg_vector"

        Returns:
            bool: Whether the operation was successful.
        """
        url = f"{self.base_url}/document"
        json = {
            "workflow_id": workflow_id,
            "unique_file_id": unique_file_id,
            "filename": filename,
            "filetype": filetype,
            "summary": summary,
            "embedding_tokens": embedding_tokens,
            "llm_tokens": llm_tokens,
            "vector_db": vector_db,
        }
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.post(url, json=json, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully inserted data for key: {unique_file_id}"
            )
            return True
        else:
            self.tool.stream_log(
                f"Error while inserting data for key: {unique_file_id} "
                f"/ {response.reason} / {response.text}",
                level=LogLevel.ERROR,
            )
            return False

    def get(self, workflow_id: str, unique_file_id: str) -> Optional[dict]:
        """Retrieves data for a document from the platform.

        Args:
            workflow_id (str): The project id.
            unique_file_id (str): The unique file id.

        Returns:
            Optional[dict]: The data for the document.
        """
        url = (
            f"{self.base_url}/document?workflow_id={workflow_id}"
            f"&unique_file_id={unique_file_id}"
        )
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully retrieved data for key: {unique_file_id}"
            )
            return response.json()
        elif response.status_code == 404:
            self.tool.stream_log(
                f"Data not found for key: {unique_file_id}", level=LogLevel.WARN
            )
            return None
        else:
            self.tool.stream_log(
                "Error while retrieving data for key: "
                f"{unique_file_id} / {response.reason}",
                level=LogLevel.ERROR,
            )
            return None

    def delete(self, workflow_id: str, unique_file_id: str):
        """Deletes data for a document from the platform.

        Notes:
            - This method is used internally. Do not call this method directly
                unless you know what you are doing.

        Args:
            workflow_id (str): The project id.
            unique_file_id (str): The unique file id.

        Returns:
            bool: Whether the operation was successful.
        """
        url = (
            f"{self.base_url}/document?workflow_id={workflow_id}"
            f"&unique_file_id={unique_file_id}"
        )
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.delete(url, headers=headers)

        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully deleted data for key: {unique_file_id}"
            )
            return True
        else:
            self.tool.stream_log(
                "Error while deleting data for key: "
                f"{unique_file_id} / {response.reason}",
                level=LogLevel.ERROR,
            )
            return False

    def answer_prompt(
        self,
        workflow_id: str,
        embedding_type: str,
        vector_db: str,
        file_path: str,
        fsspec_obj: AbstractFileSystem,
        prompt: dict[str, str],
    ) -> dict:
        """Fetches answers from the indexed document for the passed questions.
        Args:
            workflow_id (str): The project id.
            embedding_type (str): The embedding type.
                Supported values:
                    - "Azure OpenAI"
            vector_db (str): The vector db.
                Supported values:
                    - "Postgres pg_vector"
            file_path (str): The path to the file to index.
            prompt (dict): Dict storing the key value pairs of the prompts
        Returns:
            Dict: Key value pairs of the question and the corresponding keys
        Notes:
            Sample return dict:
            {
                "status": "OK",
                "error": "",
                "cost": 746,
                "answers" : "",
                "unique_file_id" : "",
            }
        """
        result = {
            "status": "ERROR",
            "error": "",
            "cost": 0,
            "unique_file_id": "",
        }
        upload_result = self._upload_file(
            file_path=file_path, fsspec_obj=fsspec_obj
        )
        if upload_result["status"] == "ERROR":
            return upload_result
        # Load Index
        payload = {
            "workflow_id": workflow_id,
            "embedding_type": embedding_type,
            "vector_db": vector_db,
            "file_path": file_path,
            "prompt": prompt,
        }
        self.tool.stream_log("Loading index and Fetching answers")
        url = f"{self.base_url}/prompt-responder"
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.post(url, json=payload, headers=headers)
        if response.status_code == 200:
            self.tool.stream_log(
                f"Successfully fetched response for the prompt : {prompt}"
            )
            result["status"] = "OK"
            result["answers"] = response.json()["answers"]
            result["unique_file_id"] = response.json()["unique_file_id"]
            result["cost"] = response.json()["cost"]
            return result
        else:
            self.tool.stream_log(
                "Error while loading indexed file and fetching response "
                f"for the prompt: {file_path} / {response.text}",
                level="ERROR",
            )
            result["error"] = response.text
            return result



from enum import Enum


class ToolEnv:
    """Environment variables used by tools.

    The 'ToolEnv' class represents a set of environment variables that are
    commonly used by tools.

    Attributes:
        PLATFORM_API_KEY (str): Platform service API key.
        PLATFORM_HOST (str): Platform service host.
        PLATFORM_PORT (str): Platform service port.
        DATA_DIR (str): The environment variable for the tool data directory.
        EXECUTION_BY_TOOL (str): Implicitly set to 1 by the SDK if its executed
            by a tool.
    """

    PLATFORM_API_KEY = "PLATFORM_SERVICE_API_KEY"
    PLATFORM_HOST = "PLATFORM_SERVICE_HOST"
    PLATFORM_PORT = "PLATFORM_SERVICE_PORT"
    DATA_DIR = "TOOL_DATA_DIR"
    EXECUTION_BY_TOOL = "EXECUTION_BY_TOOL"


class ConnectorKeys:
    ID = "id"
    PROJECT_ID = "project_id"
    CONNECTOR_ID = "connector_id"
    TOOL_INSTANCE_ID = "tool_instance_id"
    CONNECTOR_METADATA = "connector_metadata"
    CONNECTOR_TYPE = "connector_type"


class AdapterKeys:
    ADAPTER_INSTANCE_ID = "adapter_instance_id"


class PromptStudioKeys:
    PROMPT_REGISTRY_ID = "prompt_registry_id"


class ConnectorType:
    INPUT = "INPUT"
    OUTPUT = "OUTPUT"


class LogType:
    LOG = "LOG"
    UPDATE = "UPDATE"


class LogStage:
    TOOL_RUN = "TOOL_RUN"


class LogState:
    """State of logs INPUT_UPDATE tag for update the FE input component
    OUTPUT_UPDATE tag for update the FE output component."""

    INPUT_UPDATE = "INPUT_UPDATE"
    OUTPUT_UPDATE = "OUTPUT_UPDATE"


class Connector:
    FILE_SYSTEM = "FILE_SYSTEM"
    DATABASE = "DATABASE"


class Command:
    SPEC = "SPEC"
    PROPERTIES = "PROPERTIES"
    ICON = "ICON"
    RUN = "RUN"
    VARIABLES = "VARIABLES"

    @classmethod
    def static_commands(cls) -> set[str]:
        return {cls.SPEC, cls.PROPERTIES, cls.ICON, cls.VARIABLES}


class UsageType:
    LLM_COMPLETE = "LLM_COMPLETE"
    RAG = "RAG"
    INDEXER = "INDEXER"


class LogLevel(Enum):
    DEBUG = "DEBUG"
    INFO = "INFO"
    WARN = "WARN"
    ERROR = "ERROR"
    FATAL = "FATAL"


class PropKey:
    """Keys for the properties.json of tools."""

    INPUT = "input"
    OUTPUT = "output"
    RESULT = "result"
    TYPE = "type"
    RESTRICTIONS = "restrictions"
    MAX_FILE_SIZE = "maxFileSize"
    FILE_SIZE_REGEX = r"^(\d+)\s*([KkMmGgTt]B?)$"
    ALLOWED_FILE_TYPES = "allowedFileTypes"
    FUNCTION_NAME = "functionName"

    class OutputType:
        JSON = "JSON"
        TXT = "TXT"


class ToolExecKey:
    OUTPUT_DIR = "COPY_TO_FOLDER"
    METADATA_FILE = "METADATA.json"
    INFILE = "INFILE"
    SOURCE = "SOURCE"


class MetadataKey:
    SOURCE_NAME = "source_name"
    SOURCE_HASH = "source_hash"
    WORKFLOW_ID = "workflow_id"
    EXECUTION_ID = "execution_id"
    ORG_ID = "organization_id"
    TOOL_META = "tool_metadata"
    TOOL_NAME = "tool_name"
    TOTAL_ELA_TIME = "total_elapsed_time"
    ELAPSED_TIME = "elapsed_time"
    OUTPUT = "output"
    OUTPUT_TYPE = "output_type"


class ToolSettingsKey:
    """A class representing the keys used in the tool settings.

    Attributes:
        LLM_ADAPTER_ID (str): The key for the LLM adapter ID.
        EMBEDDING_ADAPTER_ID (str): The key for the embedding adapter ID.
        VECTOR_DB_ADAPTER_ID (str): The key for the vector DB adapter ID.
    """

    LLM_ADAPTER_ID = "llmAdapterId"
    EMBEDDING_ADAPTER_ID = "embeddingAdapterId"
    VECTOR_DB_ADAPTER_ID = "vectorDbAdapterId"


class FileReaderSettings:
    FILE_READER_CHUNK_SIZE = 8192



import unittest
from io import StringIO
from unittest.mock import patch

from unstract.sdk.tool.mixin import ToolConfigHelper
from unstract.sdk.tool.stream import StreamMixin
from unstract.sdk.utils import ToolUtils


class UnstractSDKToolsStaticTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.tool = StreamMixin()

    def test_spec(self):
        spec = ToolUtils.json_to_str(
            ToolConfigHelper.spec(spec_file="config/tool_spec.json")
        )
        self.assertIsNotNone(spec)

    def test_stream_spec(self):
        spec = ToolUtils.json_to_str(
            ToolConfigHelper.spec(spec_file="config/tool_spec.json")
        )
        captured_output = StringIO()
        with patch("sys.stdout", new=captured_output):
            self.tool.stream_spec(spec)
        captured_output_str = captured_output.getvalue()
        # print(captured_output_str)
        self.assertIn("SPEC", captured_output_str)

    def test_properties(self):
        properties = ToolUtils.json_to_str(
            ToolConfigHelper.properties(
                properties_file="config/tool_properties.json"
            )
        )
        self.assertIsNotNone(properties)

    def test_stream_properties(self):
        properties = ToolUtils.json_to_str(
            ToolConfigHelper.properties(
                properties_file="config/tool_properties.json"
            )
        )
        captured_output = StringIO()
        with patch("sys.stdout", new=captured_output):
            self.tool.stream_properties(properties)
        captured_output_str = captured_output.getvalue()
        # print(captured_output_str)
        self.assertIn("PROPERTIES", captured_output_str)

    def test_icon(self):
        icon = ToolConfigHelper.icon(icon_file="config/icon.svg")
        self.assertIsNotNone(icon)

    def test_stream_icon(self):
        icon = ToolConfigHelper.icon(icon_file="config/icon.svg")
        captured_output = StringIO()
        with patch("sys.stdout", new=captured_output):
            self.tool.stream_icon(icon)
        captured_output_str = captured_output.getvalue()
        # print(captured_output_str)
        self.assertIn("ICON", captured_output_str)


if __name__ == "__main__":
    unittest.main()



from typing import Any

import requests
from unstract.connectors.constants import Common
from unstract.connectors.databases import connectors
from unstract.connectors.databases.unstract_db import UnstractDB
from unstract.sdk.constants import Connector, ConnectorKeys, LogLevel
from unstract.sdk.platform import PlatformBase
from unstract.sdk.tool.base import BaseTool


class ToolDB(PlatformBase):
    """Class to handle DB connectors for Unstract Tools.

    Notes:
        - PLATFORM_SERVICE_API_KEY environment variable is required.
    """

    def __init__(
        self,
        tool: BaseTool,
        platform_host: str,
        platform_port: str,
    ) -> None:
        """
        Args:
            tool (AbstractTool): Instance of AbstractTool
            platform_host (str): Host of platform service
            platform_port (str): Port of platform service

        Notes:
            - PLATFORM_SERVICE_API_KEY environment variable is required.
            - The platform_host and platform_port are the
                host and port of the platform service.
        """
        super().__init__(
            tool=tool, platform_host=platform_host, platform_port=platform_port
        )
        self.db_connectors = connectors

    def get_engine(self, tool_instance_id: str) -> Any:
        """Get DB engine
            1. Get the connection settings from platform service
            using the tool_instance_id
            2. Create UnstractFileSystem based object using the settings
                2.1 Find the type of the database (From Connector Registry)
                2.2 Create the object using the type
                (derived class of UnstractFileSystem)
            3. Send Object.get_fsspec_fs() to the caller

        Args:
            tool_instance_id (str): tool Instance Id

        Returns:
            Any: _description_
        """

        url = f"{self.base_url}/connector_instance/{Connector.DATABASE}"
        query_params = {
            ConnectorKeys.TOOL_INSTANCE_ID: tool_instance_id,
        }
        headers = {"Authorization": f"Bearer {self.bearer_token}"}
        response = requests.get(url, headers=headers, params=query_params)
        if response.status_code == 200:
            connector_data: dict[str, Any] = response.json()
            connector_id = connector_data.get(ConnectorKeys.CONNECTOR_ID)
            connector_instance_id = connector_data.get(ConnectorKeys.ID)
            settings = connector_data.get(ConnectorKeys.CONNECTOR_METADATA)
            self.tool.stream_log(
                "Successfully retrieved connector settings "
                f"for connector: {connector_instance_id}"
            )
            if connector_id in self.db_connectors:
                connector = self.db_connectors[connector_id][Common.METADATA][
                    Common.CONNECTOR
                ]
                connector_class: UnstractDB = connector(settings)
                return connector_class.get_engine()
            else:
                self.tool.stream_log(
                    f"Engine not found for connector: {connector_id}",
                    level=LogLevel.ERROR,
                )
                return None

        elif response.status_code == 404:
            self.tool.stream_log(
                f"Connector not found for tool instance {tool_instance_id}",
                level=LogLevel.ERROR,
            )
            return None

        else:
            self.tool.stream_log(
                (
                    f"Error while retrieving connector "
                    "for tool instance: "
                    f"{tool_instance_id} / {response.reason}"
                ),
                level=LogLevel.ERROR,
            )
            return None



import logging
from typing import Any, Callable, Optional, Union

import tiktoken
from llama_index import ServiceContext as LlamaIndexServiceContext
from llama_index.callbacks import CallbackManager, TokenCountingHandler
from llama_index.core.embeddings.base import BaseEmbedding
from llama_index.llms import LLM
from llama_index.llms.utils import LLMType
from transformers import AutoTokenizer
from unstract.sdk.utils.usage_handler import UsageHandler

logger = logging.getLogger(__name__)


class ServiceContext:
    """Class representing the UNServiceContext.

    Use this over the default service context of llama index

    This class provides a static method to get the service context for
    UNstract Tools. The service context includes a tokenizer, token counter,
    usage handler, and  callback manager.

    Attributes:
        None

    Methods:
        get_service_context: Returns the service context for UNstract Tools.

    Example:
        service_context = UNServiceContext.
                            get_service_context(
                                workflow_id="123",
                                execution_id="456",
                                llm="default",
                                embed_model="default")
    """

    @staticmethod
    def get_service_context(
        platform_api_key: str,
        workflow_id: str = "",
        execution_id: str = "",
        llm: Optional[LLMType] = None,
        embed_model: Optional[Any] = None,
        **kwargs: Any,
    ) -> LlamaIndexServiceContext:
        """Returns the service context for UNstract Tools.

        Parameters:
            workflow_id (str): The workflow ID. Default is an empty string.
            execution_id (str): The execution ID. Default is an empty string.
            llm (Optional[LLMType]): The LLM type. Default is None.
            embed_model (Optional[Any]): The embedding model. Default is None.

        Returns:
            ServiceContext: The service context for UNstract Tools.

        Example:
            service_context = UNServiceContext.get_service_context(
                workflow_id="123",
                execution_id="456",
                llm="default",
                embed_model="default"
            )
        """
        if llm:
            tokenizer = ServiceContext.get_tokenizer(llm)
        elif embed_model:
            tokenizer = ServiceContext.get_tokenizer(embed_model)

        token_counter = TokenCountingHandler(tokenizer=tokenizer, verbose=True)
        usage_handler = UsageHandler(
            token_counter=token_counter,
            platform_api_key=platform_api_key,
            llm_model=llm,
            embed_model=embed_model,
            workflow_id=workflow_id,
            execution_id=execution_id,
        )

        callback_manager = CallbackManager(
            handlers=[token_counter, usage_handler]
        )
        return LlamaIndexServiceContext.from_defaults(
            llm=llm,
            embed_model=embed_model,
            callback_manager=callback_manager,
            **kwargs,
        )

    @staticmethod
    def get_tokenizer(
        model: Optional[Union[LLM, BaseEmbedding, None]],
        fallback_tokenizer: Callable[[str], list] = tiktoken.encoding_for_model(
            "gpt-3.5-turbo"
        ).encode,
    ) -> Callable[[str], list]:
        """Returns a tokenizer function based on the provided model.

        Args:
            model (Optional[Union[LLM, BaseEmbedding]]): The model to use for
            tokenization.

        Returns:
            Callable[[str], List]: The tokenizer function.

        Raises:
            OSError: If an error occurs while loading the tokenizer.
        """

        try:
            if isinstance(model, LLM):
                model_name: str = model.metadata.model_name
            elif isinstance(model, BaseEmbedding):
                model_name = model.model_name

            tokenizer: Callable[[str], list] = AutoTokenizer.from_pretrained(
                model_name
            ).encode
            return tokenizer
        except OSError as e:
            logger.warning(str(e))
            return fallback_tokenizer



import logging
from typing import Union

from llama_index.vector_stores.types import BasePydanticVectorStore, VectorStore
from unstract.adapters.constants import Common
from unstract.adapters.vectordb import adapters
from unstract.adapters.vectordb.constants import VectorDbConstants
from unstract.sdk.adapters import ToolAdapter
from unstract.sdk.constants import LogLevel, ToolEnv, ToolSettingsKey
from unstract.sdk.exceptions import SdkException
from unstract.sdk.platform import PlatformHelper
from unstract.sdk.tool.base import BaseTool

logger = logging.getLogger(__name__)


class ToolVectorDB:
    """Class to handle VectorDB for Unstract Tools."""

    def __init__(self, tool: BaseTool, tool_settings: dict[str, str] = {}):
        self.tool = tool
        self.vector_db_adapters = adapters
        self.vector_db_adapter_instance_id = tool_settings.get(
            ToolSettingsKey.VECTOR_DB_ADAPTER_ID
        )

    def __get_org_id(self) -> str:
        platform_helper = PlatformHelper(
            tool=self.tool,
            platform_host=self.tool.get_env_or_die(ToolEnv.PLATFORM_HOST),
            platform_port=int(self.tool.get_env_or_die(ToolEnv.PLATFORM_PORT)),
        )
        # fetch org id from bearer token
        platform_details = platform_helper.get_platform_details()
        if not platform_details:
            # Errors are logged by the SDK itself
            raise SdkException("Error getting platform details")
        account_id = platform_details.get("organization_id")
        return account_id

    def get_vector_db(
        self, adapter_instance_id: str, embedding_dimension: int
    ) -> Union[BasePydanticVectorStore, VectorStore, None]:
        adapter_instance_id = (
            adapter_instance_id
            if adapter_instance_id
            else self.vector_db_adapter_instance_id
        )
        if adapter_instance_id is not None:
            try:
                vector_db_config = ToolAdapter.get_adapter_config(
                    self.tool, adapter_instance_id
                )
                vector_db_adapter_id = vector_db_config.get(Common.ADAPTER_ID)
                if vector_db_adapter_id in self.vector_db_adapters:
                    vector_db_adapter = self.vector_db_adapters[
                        vector_db_adapter_id
                    ][Common.METADATA][Common.ADAPTER]
                    vector_db_metadata = vector_db_config.get(
                        Common.ADAPTER_METADATA
                    )
                    org = self.__get_org_id()
                    # Adding the collection prefix and embedding type
                    # to the metadata
                    vector_db_metadata[VectorDbConstants.VECTOR_DB_NAME] = org
                    vector_db_metadata[
                        VectorDbConstants.EMBEDDING_DIMENSION
                    ] = embedding_dimension

                    vector_db_adapter_class = vector_db_adapter(
                        vector_db_metadata
                    )
                    return vector_db_adapter_class.get_vector_db_instance()
                else:
                    return None
            except Exception as e:
                self.tool.stream_log(
                    log=f"Unable to get vector_db {adapter_instance_id}: {e}",
                    level=LogLevel.ERROR,
                )
                return None



import unittest
from typing import Any

from unstract.sdk.documents import ToolDocs
from unstract.sdk.tool.base import BaseTool


class UnstractToolDocsTest(unittest.TestCase):
    class MockTool(BaseTool):
        def run(
            self,
            params: dict[str, Any] = {},
            settings: dict[str, Any] = {},
            workflow_id: str = "",
        ) -> None:
            # self.stream_log("Mock tool running")
            pass

    @classmethod
    def setUpClass(cls):
        cls.tool = cls.MockTool()

    @unittest.skip("Skipping")
    def test_insert_get_delete(self) -> None:
        docs = ToolDocs(
            tool=self.tool, platform_host="http://localhost", platform_port=3001
        )
        result = docs.insert(
            workflow_id="test",
            unique_file_id="test_unique_file_id2",
            filename="test_filename",
            filetype="test_filetype",
            summary="test_summary",
            embedding_tokens=0,
            llm_tokens=0,
            vector_db="Postgres pg_vector",
        )
        self.assertTrue(result)
        result = docs.get(
            workflow_id="test", unique_file_id="test_unique_file_id2"
        )
        print(result)
        self.assertIsNotNone(result)
        result = docs.delete(
            workflow_id="test", unique_file_id="test_unique_file_id2"
        )
        self.assertTrue(result)
        result = docs.get(
            workflow_id="test", unique_file_id="test_unique_file_id2"
        )
        self.assertIsNone(result)

    # def test_indexer(self) -> None:
    #     docs = ToolDocs(
    #         tool=self.tool, platform_host="http://localhost", platform_port=3001  # noqa: E501
    #     )
    #     result = docs.index_file(
    #         workflow_id="test2",
    #         embedding_type="Azure OpenAI",
    #         vector_db="Postgres pg_vector",
    #         file_path="/mnt/unstract/fs_input/files/Suriya cv.pdf",
    #         overwrite=True,
    #     )
    #     print(result)
    #     self.assertTrue(result)


if __name__ == "__main__":
    unittest.main()



import logging
import time
from typing import Any, Optional

from llama_index.llms import LLM
from llama_index.llms.base import CompletionResponse
from unstract.adapters.constants import Common
from unstract.adapters.llm import adapters
from unstract.adapters.llm.llm_adapter import LLMAdapter
from unstract.sdk.adapters import ToolAdapter
from unstract.sdk.constants import LogLevel, ToolSettingsKey
from unstract.sdk.tool.base import BaseTool
from unstract.sdk.utils.service_context import ServiceContext

logger = logging.getLogger(__name__)


class ToolLLM:
    """Class to handle LLMs for Unstract Tools."""

    def __init__(
        self,
        tool: BaseTool,
        tool_settings: dict[str, str] = {},
    ):
        """

        Notes:
            - "Azure OpenAI" : Environment variables required
            OPENAI_API_KEY,OPENAI_API_BASE, OPENAI_API_VERSION,
            OPENAI_API_ENGINE, OPENAI_API_MODEL

        Args:
            tool (AbstractTool): Instance of AbstractTool
        """
        self.tool = tool
        self.max_tokens = 1024 * 4
        self.llm_adapters = adapters
        self.llm_adapter_instance_id = tool_settings.get(
            ToolSettingsKey.LLM_ADAPTER_ID
        )

    @staticmethod
    def run_completion(
        llm: LLM,
        platform_api_key: str,
        prompt: str,
        retries: int = 3,
        **kwargs: Any,
    ) -> Optional[dict[str, Any]]:
        ServiceContext.get_service_context(
            platform_api_key=platform_api_key, llm=llm
        )
        for i in range(retries):
            try:
                response: CompletionResponse = llm.complete(prompt, **kwargs)
                result = {
                    "response": response,
                }

                return result
            except Exception as e:
                if i == retries - 1:
                    raise e
                time.sleep(5)
        return None

    def get_llm(
        self, adapter_instance_id: Optional[str] = None
    ) -> Optional[LLM]:
        """Returns the LLM object for the tool.

        Returns:
            Optional[LLM]: The LLM object for the tool.
            (llama_index.llms.base.LLM)
        """
        adapter_instance_id = (
            adapter_instance_id
            if adapter_instance_id
            else self.llm_adapter_instance_id
        )
        # Support for get_llm using adapter_instance_id
        if adapter_instance_id is not None:
            try:
                llm_config_data = ToolAdapter.get_adapter_config(
                    self.tool, adapter_instance_id
                )
                llm_adapter_id = llm_config_data.get(Common.ADAPTER_ID)
                if llm_adapter_id in self.llm_adapters:
                    llm_adapter = self.llm_adapters[llm_adapter_id][
                        Common.METADATA
                    ][Common.ADAPTER]
                    llm_metadata = llm_config_data.get(Common.ADAPTER_METADATA)
                    llm_adapter_class: LLMAdapter = llm_adapter(llm_metadata)
                    llm_instance: Optional[
                        LLM
                    ] = llm_adapter_class.get_llm_instance()
                    return llm_instance
                else:
                    return None
            except Exception as e:
                self.tool.stream_log(
                    log=f"Unable to get llm instance: {e}", level=LogLevel.ERROR
                )
                return None
        else:
            logger.error("The adapter_instance_id parameter is None")
            return None

    def get_max_tokens(self, reserved_for_output: int = 0) -> int:
        """Returns the maximum number of tokens that can be used for the LLM.

        Args:
            reserved_for_output (int): The number of tokens reserved for the
                                        output.
                The default is 0.

            Returns:
                int: The maximum number of tokens that can be used for the LLM.
        """
        return self.max_tokens - reserved_for_output



import logging
from typing import Any

from unstract.sdk.utils import ToolUtils

logger = logging.getLogger(__name__)


class ToolConfigHelper:
    """Helper class to handle static commands for tools."""

    @staticmethod
    def spec(spec_file: str = "config/spec.json") -> dict[str, Any]:
        """Returns the JSON schema for the tool settings.

        Args:
            spec_file (str): The path to the JSON schema file.
            The default is config/spec.json.
        Returns:
            str: The JSON schema of the tool.
        """
        return ToolUtils.load_json(spec_file)

    @staticmethod
    def properties(
        properties_file: str = "config/properties.json",
    ) -> dict[str, Any]:
        """Returns the properties of the tool.

        Args:
            properties_file (str): The path to the properties file.
            The default is config/properties.json.
        Returns:
            str: The properties of the tool.
        """
        return ToolUtils.load_json(properties_file)

    @staticmethod
    def variables(
        variables_file: str = "config/runtime_variables.json",
    ) -> dict[str, Any]:
        """Returns the JSON schema of the runtime variables.

        Args:
            variables_file (str): The path to the JSON schema file.
            The default is config/runtime_variables.json.
        Returns:
            str: The JSON schema for the runtime variables.
        """

        try:
            return ToolUtils.load_json(variables_file)
        # Allow runtime variables definition to be optional
        except FileNotFoundError:
            logger.info("No runtime variables defined for tool")
            return {}

    @staticmethod
    def icon(icon_file: str = "config/icon.svg") -> str:
        """Returns the icon of the tool.

        Args:
            icon_file (str): The path to the icon file.
                The default is config/icon.svg.
        Returns:
            str: The icon of the tool.
        """
        with open(icon_file, encoding="utf-8") as f:
            icon = f.read()
            return icon
